spool BD\11-logSP_QUITAR_IMEI.log;

CONNECT TFC/hola;
create or replace PROCEDURE SP_QUITAR_IMEI (
   p_nummobil 	IN  TFC.USUARIS.NUMMOBIL%TYPE,
   p_codimei    IN TFC.imei.CODIMEI%TYPE,
   RSP         OUT VARCHAR2
)
/******************************************************************************
   Nombre:                 SP_QUITAR_IMEI
   Propsito:            Elimina la asociacin de un IMEI con un usuario
                   
   Parmetros entrada: 
                   -   p_nummobil  	 VARCHAR2(20) , nombre del usuario
                   -    p_codimei    VARCHAR2(15), primer imei usuario,                                        
   Parmetros salida:  RSP
   Retorna:             'OK' si la ejecucin ha finalizado con xito
                        'ERROR: tipo de error' si se ha producido un error
******************************************************************************/


IS
   E_CAMPO_NULO             EXCEPTION;
   E_USUARIO_NO_EXISTE      EXCEPTION;
   E_IMEI_NO_EXISTE         EXCEPTION;
   E_IMEI_NO_DISPONIBLE     EXCEPTION; 
   existe_imei     VARCHAR2(1);
   existe_usuari   VARCHAR2(1);
   param_entrada   VARCHAR2(500);
   err_num         NUMBER;
   err_msg         VARCHAR2(100);
   p_idusuari      NUMBER;

   BEGIN

     param_entrada := 'P_NUMMOBIL: '||p_nummobil||', P_CODIMEI:'||p_codimei;

     --Se comprueba que ambos campos tengan datos.
     IF (TRIM(p_nummobil)     IS NULL)
     THEN
         RAISE E_CAMPO_NULO;
      END IF;
   IF (TRIM(p_codimei)     IS NULL)
     THEN
         RAISE E_CAMPO_NULO;
      END IF;

     --Verifica exista al que se le quiere quitar un IMEI exista.
     BEGIN

       SELECT 'S'
         INTO existe_usuari
         FROM TFC.USUARIS
        WHERE NUMMOBIL  = p_nummobil
          AND ROWNUM  = 1;     
      EXCEPTION
        WHEN NO_DATA_FOUND THEN
           RAISE E_USUARIO_NO_EXISTE;
      END;
 
--Verifica que el IMEI que se quiere borrar est asignado a ese usuario.    
BEGIN
       
        SELECT TFC.USUARIS.IDUSUARI
        INTO p_idusuari
         FROM TFC.USUARIS
        WHERE TFC.USUARIS.NUMMOBIL  = p_nummobil
         AND ROWNUM  = 1;
        
        
       SELECT 'S'
        INTO existe_imei
        FROM TFC.IMEI
        WHERE CODIMEI  = p_codimei
        AND idusuariimei = p_idusuari
        AND ROWNUM  = 1;
        EXCEPTION
        WHEN NO_DATA_FOUND THEN
          RAISE  E_IMEI_NO_EXISTE;
       
           
      END;


      --Si ha pasado todas las pruebas de validacin realiza el borrado

  
    
      UPDATE TFC.imei
      set idusuariimei = NULL
      where tfc.imei.codimei = p_codimei;
      RSP := 'OK';

    

      INSERT INTO TFC.LOGS (datalog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
           VALUES(SYSDATE,'SP_QUITAR_IMEI', param_entrada, 'RSP: '||RSP);

      COMMIT;

  EXCEPTION
    WHEN E_USUARIO_NO_EXISTE THEN
      RSP := 'ERROR: EL USUARIO NO EST DADO DE ALTA EN EL SISTEMA';

      INSERT INTO TFC.LOGS (datalog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
         VALUES(SYSDATE, 'SP_QUITAR_IMEI', param_entrada, 'RSP: '||RSP);

      COMMIT;

    WHEN E_CAMPO_NULO THEN
      RSP := 'ERROR: UNO DE LOS CAMPOS EST EN BLANCO';

      INSERT INTO TFC.LOGS (datalog, PROCEDIMENT, PARAMSENTRADA, PARAMSSortida)
         VALUES(SYSDATE, 'SP_QUITAR_IMEI', param_entrada, 'RSP: '||RSP);

      COMMIT;

    WHEN E_IMEI_NO_EXISTE THEN
      RSP := 'ERROR: EL IMEI NO EXISTE';

      INSERT INTO LOGS (DATALOG, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
         VALUES(SYSDATE, 'SP_QUITAR_IMEI', param_entrada, 'RSP: '||RSP);

      COMMIT;

    WHEN OTHERS THEN
      err_num := SQLCODE;
      err_msg := SUBSTR(SQLERRM, 1, 200);

      RSP:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

      INSERT INTO tfc.LOGS (datalog, PROCEDIMENT, PARAMSENTRADA, paramssortida)
         VALUES(SYSDATE, 'SP_QUITAR_IMEI', param_entrada, 'RSP: '||RSP);

      COMMIT;


END SP_QUITAR_IMEI;
/
spool off;