spool BD\12-logSP_MODIF_USUARI.log;

CONNECT TFC/hola;

create or replace PROCEDURE SP_MODIF_USUARI (
   p_nummobil   	      IN  TFC.USUARIS.NUMMOBIL%TYPE,
   p_nomusuari      	  IN  TFC.USUARIS.NOMUSUARI%TYPE,
   p_cognom1usuari  	  IN  TFC.USUARIS.COGNOM1USUARI%TYPE,
   p_cognom2usuari  	  IN  TFC.USUARIS.COGNOM2USUARI%TYPE,
   p_operadortelefonic  IN  TFC.USUARIS.OPERADORTELEFONIC%TYPE,
   p_emailusuari		    IN  TFC.USUARIS.EMAILUSUARI%TYPE,
   RSP         		OUT VARCHAR2
)
IS
/******************************************************************************
   Nombre:                 SP_MODIF_USUARI
   Propsito:            Modifica los datos de un usuario a la tabla USUARIS
                   como nombre, apellidos, operador o email
   Parmetros entrada: 
                   -   p_nummobil     NUMBER        , nmero de telfono del usuario        
                   -   p_nomusuari  	 VARCHAR2(20) , nombre del usuario
                   -   p_cognom1usuari VARCHAR2(20) , primer apellido usuario
                   -   p_cognom2usuari VARCHAR2(20) , segundo apellido usuario
                   -   p_operadortelefonic VARCHAR2(20) , operador telefnico
                  -    p_emailusuari		VARCHAR2(40) , email usuario
                
   Parmetros salida:  RSP
   Retorna:             'OK' si la ejecucin ha finalizado con xito
                        'ERROR: tipo de error' si se ha producido un error
******************************************************************************/

   E_CAMPO_NULO            EXCEPTION;
   E_USUARIO_NO_EXISTE     EXCEPTION;
   

   existe        VARCHAR2(1);
   param_entrada VARCHAR2(500);
   err_num       NUMBER;
   err_msg       VARCHAR2(100);


   BEGIN

     param_entrada := 'P_NUMMOBIL: '||p_nummobil||', P_NOMUSUARI:'||p_nomusuari||', P_COGNOM1USUARI:'||p_cognom1usuari||', P_COGNOM2USUARI:'||p_cognom2usuari||', P_OPERADORTELEFONIC:'||p_operadortelefonic||',  P_EMAILUSUARI:'||p_emailusuari;

     --Verifica que todos los campos obligatorios tengan valor
     IF (TRIM(p_nummobil)     IS NULL)
     THEN
         RAISE E_CAMPO_NULO;
      END IF;

     --Verifica que el usario es de alta en el sistema
     BEGIN

       SELECT 'S'
         INTO EXISTE
         FROM TFC.USUARIS
        WHERE NUMMOBIL  = p_nummobil
          AND ROWNUM  = 1;
      EXCEPTION
        WHEN NO_DATA_FOUND THEN
        RAISE E_USUARIO_NO_EXISTE;
      END;

      --Si ha pasado todas las pruebas de validacin realiza la modificacin

      UPDATE TFC.USUARIS
      SET TFC.USUARIS.NOMUSUARI = nvl(p_nomusuari,NOMUSUARI),
      TFC.USUARIS.COGNOM1USUARI = nvl(p_cognom1usuari,COGNOM1USUARI),
      TFC.USUARIS.COGNOM2USUARI = nvl(p_cognom2usuari,COGNOM2USUARI),
      TFC.USUARIS.OPERADORTELEFONIC = nvl(p_operadortelefonic,OPERADORTELEFONIC),
      TFC.USUARIS.EMAILUSUARI = nvl(p_emailusuari,EMAILUSUARI)
      where TFC.USUARIS.NUMMOBIL = p_nummobil;
      
      RSP := 'OK';
    
      
      INSERT INTO TFC.LOGS (datalog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
           VALUES(SYSDATE,'SP_MODIF_USUARI', param_entrada, 'RSP: '||RSP);

      COMMIT;

  EXCEPTION
    WHEN E_USUARIO_NO_EXISTE THEN
      RSP := 'ERROR: EL USUARIO NO EST DADO DE ALTA EN EL SISTEMA';

      INSERT INTO TFC.LOGS (datalog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
         VALUES(SYSDATE, 'SP_MODIF_USUARI', param_entrada, 'RSP: '||RSP);

      COMMIT;

    WHEN E_CAMPO_NULO THEN
      RSP := 'ERROR: EL CAMPO DEL NMERO DE MVIL EST EN BLANCO';

      INSERT INTO TFC.LOGS (datalog, PROCEDIMENT, PARAMSENTRADA, PARAMSSortida)
         VALUES(SYSDATE, 'SP_MODIF_USUARI', param_entrada, 'RSP: '||RSP);

      COMMIT;

   

    WHEN OTHERS THEN
      err_num := SQLCODE;
      err_msg := SUBSTR(SQLERRM, 1, 200);

      RSP:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

      INSERT INTO tfc.LOGS (datalog, PROCEDIMENT, PARAMSENTRADA, paramssortida)
         VALUES(SYSDATE, 'SP_MODIF_USUARI', param_entrada, 'RSP: '||RSP);

      COMMIT;


END SP_MODIF_USUARI;
/
spool off;