spool BD\13-logSP_BAJA_USUARI.log;

CONNECT TFC/hola;

create or replace PROCEDURE SP_BAJA_USUARI (
   p_nummobil 	IN  TFC.USUARIS.NUMMOBIL%TYPE,
  
   
   RSP         		OUT VARCHAR2
)
IS
/******************************************************************************
   Nombre:                 SP_BAJA_USUARI
   Propsito:            Elimina un usuario a la tabla USUARIS
                   
   Parmetros entrada: 
                   -   p_nummobil  	 NUMBER , Nmero de telfono del usuario
                   
   Parmetros salida:  RSP
   Retorna:             'OK' si la ejecucin ha finalizado con xito
                        'ERROR: tipo de error' si se ha producido un error
******************************************************************************/

   E_CAMPO_NULO         EXCEPTION;
   E_USUARIO_NO_EXISTE      EXCEPTION;
   

   existe        VARCHAR2(1);
   p_idusuari      NUMBER;
   param_entrada VARCHAR2(500);
   err_num       NUMBER;
   err_msg       VARCHAR2(100);


   BEGIN

     param_entrada := 'P_NUMMOBIL: '||p_nummobil;

     --Verifica que todos los campos obligatorios tengan valor
     IF (TRIM(p_nummobil)     IS NULL)
     THEN
         RAISE E_CAMPO_NULO;
      END IF;

     --Verifica que el Usuario exista
     BEGIN

       SELECT 'S'
         INTO EXISTE
         FROM TFC.USUARIS
        WHERE NUMMOBIL  = p_nummobil
          AND ROWNUM  = 1;

      EXCEPTION
        WHEN NO_DATA_FOUND THEN
        RAISE E_USUARIO_NO_EXISTE;
      END;
           
           
           
      
      --Si ha pasado todas las pruebas de validacin realiza el borrado

  SELECT TFC.USUARIS.IDUSUARI
        INTO p_idusuari
         FROM TFC.USUARIS
        WHERE TFC.USUARIS.NUMMOBIL  = p_nummobil
         AND ROWNUM  = 1;
         
      UPDATE TFC.imei
      set idusuariimei = NULL
      where tfc.imei.idusuariimei = p_idusuari;
      
      
      DELETE FROM TFC.usuaris 
         WHERE TFC.USUARIS.NUMMOBIL  = p_nummobil;
        
      RSP := 'OK';

   

      INSERT INTO TFC.LOGS (datalog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
           VALUES(SYSDATE,'SP_BAJA_USUARI', param_entrada, 'RSP: '||RSP);

      COMMIT;

  EXCEPTION
    WHEN E_USUARIO_NO_EXISTE THEN
      RSP := 'ERROR:EL USUARIO NO EST DE ALTA EN EL SISTEMA';

      INSERT INTO TFC.LOGS (datalog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
         VALUES(SYSDATE, 'SP_BAJA_USUARI', param_entrada, 'RSP: '||RSP);

      COMMIT;

    WHEN E_CAMPO_NULO THEN
      RSP := 'ERROR: EL CAMPO DEL NMERO DE MVIL EST EN BLANCO';

      INSERT INTO TFC.LOGS (datalog, PROCEDIMENT, PARAMSENTRADA, PARAMSSortida)
         VALUES(SYSDATE, 'SP_BAJA_USUARI', param_entrada, 'RSP: '||RSP);

      COMMIT;

   
    WHEN OTHERS THEN
      err_num := SQLCODE;
      err_msg := SUBSTR(SQLERRM, 1, 200);

      RSP:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

      INSERT INTO tfc.LOGS (datalog, PROCEDIMENT, PARAMSENTRADA, paramssortida)
         VALUES(SYSDATE, 'SP_BAJA_DESENVOLUPADOR', param_entrada, 'RSP: '||RSP);

      COMMIT;


END SP_BAJA_USUARI;
/
spool off;