spool BD\14-logSP_ALTA_DESENVOLUPADOR.log

CONNECT TFC/hola;

create or replace PROCEDURE SP_ALTA_DESENVOLUPADOR (
   p_nomempresa 	IN  TFC.DESENVOLUPADORS.NOMEMPRESA%TYPE,
   p_nomreplegal  	IN  TFC.DESENVOLUPADORS.NOMREPLEGAL%TYPE,
   p_cognom1replegal  	IN  TFC.DESENVOLUPADORS.COGNOM1REPLEGAL%TYPE,
   p_cognom2replegal  	IN  TFC.DESENVOLUPADORS.COGNOM2REPLEGAL%TYPE,
   p_codpaisregistre  	IN  TFC.DESENVOLUPADORS.CODPAISREGISTRE%TYPE,
   p_adreaoficina  	IN  TFC.DESENVOLUPADORS.ADREAOFICINA%TYPE,
   p_telefon		IN  TFC.DESENVOLUPADORS.TELEFON%TYPE,
   RSP         		OUT VARCHAR2
)
IS
/******************************************************************************
   Nombre:                 SP_ALTA_DESENVOLUPADOR
   Propsito:            introduce un desarrollador a la tabla DESENVOLUPADORS
                   
   Parmetros entrada:   -	p_nomempresa 		VARCHAR2(20): 	Nombre de la empresa desarrolladora.
                         -	p_nomreplegal 		VARCHAR2(20): 	Nombre del representante legal.
                         -	p_cognom1replegal 	VARCHAR2(20): 	Primer apellido del representante legal.
			 -	p_cognom2replegal 	VARCHAR2(20): 	Segundo apellido del representante legal.
                         -	p_codpaisregistre  	VARCHAR(2): 	Cdigo del pas de la empresa.
			 -	p_adreaoficina  	VARCHAR2(30): 	Direccin de la empresa.
			 -	p_telefon		NUMBER:		Nmero de telfono de la empresa	
                                                               
   Parmetros salida:  RSP
   Retorna:             'OK' si la ejecucin ha finalizado con xito
                        'ERROR: tipo de error' si se ha producido un error
******************************************************************************/

   E_CAMPO_NULO         EXCEPTION;
   E_DESENVOLUPADOR_EXISTE      EXCEPTION;
   E_PAIS_NO_EXISTE EXCEPTION;

   existepais    VARCHAR2(1);
   existe        VARCHAR2(1); 
   param_entrada VARCHAR2(500);
   err_num       NUMBER;
   err_msg       VARCHAR2(100);


   BEGIN

     param_entrada := 'P_NOMEMPRESA: '||p_nomempresa||', P_NOMREPLEGAL:'||p_nomreplegal||', P_COGNOM1REPLEGAL:'||p_cognom1replegal||
                      ', P_COGNOM1REPLEGAL:'||p_cognom1replegal||', P_CODPAISREGISTRE:'||p_codpaisregistre||', 
			P_ADREAOFICINA:'||p_adreaoficina||',  P_TELEFON:'||p_telefon;

     --Verifica que todos los campos obligatorios tengan valor
     IF (TRIM(p_nomempresa)     IS NULL)
     THEN
         RAISE E_CAMPO_NULO;
      END IF;

     --Verifica que el registro no se haya insertado previmente con otro ID
     BEGIN

       SELECT 'S'
         INTO EXISTE
         FROM TFC.DESENVOLUPADORS
        WHERE NOMEMPRESA  = p_nomempresa
          AND ROWNUM  = 1;

        RAISE E_DESENVOLUPADOR_EXISTE;
      EXCEPTION
        WHEN NO_DATA_FOUND THEN
           NULL;
       END;
       
       BEGIN
       SELECT 'S'
         INTO EXISTEPAIS
         FROM TFC.PAISOS
        WHERE CODPAIS  = p_codpaisregistre
          AND ROWNUM  = 1;
        EXCEPTION
        WHEN NO_DATA_FOUND THEN
          RAISE  E_PAIS_NO_EXISTE;
           
           
           
      END;

      --Si ha pasado todas las pruebas de validacin realiza la insercin
      INSERT INTO DESENVOLUPADORS (NOMEMPRESA,NOMREPLEGAL,COGNOM1REPLEGAL,COGNOM2REPLEGAL,CODPAISREGISTRE,ADREAOFICINA,TELEFON)
           VALUES (p_nomempresa, p_nomreplegal,p_cognom1replegal, p_cognom2replegal, p_codpaisregistre, p_adreaoficina, p_telefon);

      RSP := 'OK';

      INSERT INTO LOGS (DATALOG, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
           VALUES(SYSDATE,'SP_ALTA_DESENVOLUPADOR', param_entrada, 'RSP: '||RSP);

      COMMIT;

  EXCEPTION
    WHEN E_DESENVOLUPADOR_EXISTE THEN
      RSP := 'ERROR: EL DESARROLLADOR YA EST DE ALTA';

      INSERT INTO LOGS (DATALOG, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
         VALUES(SYSDATE, 'SP_ALTA_DESENVOLUPADOR', param_entrada, 'RSP: '||RSP);

      COMMIT;

    WHEN E_CAMPO_NULO THEN
      RSP := 'ERROR: EL CAMPO NOMBRE DEL DESARROLADOR EST VACIO';

      INSERT INTO LOGS (DATALOG, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
         VALUES(SYSDATE, 'SP_ALTA_DESENVOLUPADOR', param_entrada, 'RSP: '||RSP);

      COMMIT;

    WHEN E_PAIS_NO_EXISTE THEN
      RSP := 'ERROR: EL PAS NO EXISTE';

      INSERT INTO LOGS (DATALOG, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
         VALUES(SYSDATE, 'SP_ALTA_DESENVOLUPADOR', param_entrada, 'RSP: '||RSP);

      COMMIT;


    WHEN OTHERS THEN
      err_num := SQLCODE;
      err_msg := SUBSTR(SQLERRM, 1, 200);

      RSP:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

      INSERT INTO LOGS (DATALOG, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
         VALUES(SYSDATE, 'SP_ALTA_DESENVOLUPADOR', param_entrada, 'RSP: '||RSP);

      COMMIT;

              
END SP_ALTA_DESENVOLUPADOR;
/
spool off;