SPOOL BD\15-logSP_MODIF_DESENVOLUPADOR.log

CONNECT TFC/hola;

create or replace PROCEDURE SP_MODIF_DESENVOLUPADOR (
   p_nomempresa 	IN  TFC.DESENVOLUPADORS.NOMEMPRESA%TYPE,
   p_nomreplegal  	IN  TFC.DESENVOLUPADORS.NOMREPLEGAL%TYPE,
   p_cognom1replegal  	IN  TFC.DESENVOLUPADORS.COGNOM1REPLEGAL%TYPE,
   p_cognom2replegal  	IN  TFC.DESENVOLUPADORS.COGNOM2REPLEGAL%TYPE,
   p_adreaoficina  	IN  TFC.DESENVOLUPADORS.ADREAOFICINA%TYPE,
   p_telefon		IN  TFC.DESENVOLUPADORS.TELEFON%TYPE,
   
   RSP         		OUT VARCHAR2
)
IS
/******************************************************************************
   Nombre:                 SP_MODIF_DESENVOLUPADOR
   Propsito:            Modifica los datos de un desarrollador en la tabla DESENVOLUPADORS
                   
   Parmetros entrada:   -	p_nomempresa 		VARCHAR2(20): 	Nombre de la empresa desarrolladora.
                         -	p_nomreplegal 		VARCHAR2(20): 	Nombre del representante legal.
                         -	p_cognom1replegal 	VARCHAR2(20): 	Primer apellido del representante legal.
                    		 -	p_cognom2replegal 	VARCHAR2(20): 	Segundo apellido del representante legal.
                      	 -	p_adreaoficina  	VARCHAR2(30): 	Direccin de la empresa.
                         - 	p_telefon		NUMBER:		Nmero de telfono de la empresa	                                                            
   Parmetros salida:  RSP
   Retorna:             'OK' si la ejecucin ha finalizado con xito
                        'ERROR: tipo de error' si se ha producido un error
                        'ERROR' El Desarrollador no existe en la base de datos
******************************************************************************/

   E_CAMPO_NULO                    EXCEPTION;
   E_DESENVOLUPADOR_NO_EXISTE      EXCEPTION;
   existe                          VARCHAR2(1);
   param_entrada                   VARCHAR2(500);
   err_num                         NUMBER;
   err_msg                         VARCHAR2(100);


   BEGIN

     param_entrada := 'P_NOMEMPRESA: '||p_nomempresa||', P_NOMREPLEGAL:'||p_nomreplegal||', P_COGNOM1REPLEGAL:'||p_cognom1replegal||', P_COGNOM2REPLEGAL:'||p_cognom2replegal||', P_ADREAOFICINA:'||p_adreaoficina||',  P_TELEFON:'||p_telefon;

     --Verifica que todos los campos obligatorios tengan valor
     IF (TRIM(p_nomempresa)     IS NULL)
     THEN
         RAISE E_CAMPO_NULO;
      END IF;

     --Verifica que el registro no se haya insertado previmente con otro ID
     BEGIN

       SELECT 'S'
         INTO EXISTE
         FROM TFC.DESENVOLUPADORS
        WHERE NOMEMPRESA  = p_nomempresa
          AND ROWNUM  = 1;

        --RAISE E_DESENVOLUPADOR_EXISTE;
      EXCEPTION
        WHEN NO_DATA_FOUND THEN
           RAISE E_DESENVOLUPADOR_NO_EXISTE;
      END;
      
      
      --Si el desarrollador a modificar de baja existe realiza el borrado
UPDATE TFC.DESENVOLUPADORS
   SET 
   TFC.DESENVOLUPADORS.NOMREPLEGAL = p_nomreplegal,
   TFC.DESENVOLUPADORS.COGNOM1REPLEGAL = p_cognom1replegal,
   TFC.DESENVOLUPADORS.COGNOM2REPLEGAL = p_cognom2replegal,
   TFC.DESENVOLUPADORS.ADREAOFICINA = p_adreaoficina,
   TFC.DESENVOLUPADORS.TELEFON = p_telefon
   WHERE NOMEMPRESA  = p_nomempresa;

      RSP := 'OK';
      INSERT INTO TFC.LOGS (datalog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
           VALUES(SYSDATE,'SP_MODIF_DESENVOLUPADOR', param_entrada, 'RSP: '||RSP);
      
   
   
  
      COMMIT;

  EXCEPTION
   WHEN E_DESENVOLUPADOR_NO_EXISTE THEN
   
      RSP := 'ERROR: EL DESARROLLADOR NO EST DADO DE ALTA EN LA PLATAFORMA';

      INSERT INTO TFC.LOGS (datalog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
         VALUES(SYSDATE, 'SP_MODIF_DESENVOLUPADOR', param_entrada, 'RSP: '||RSP);
   

   

      COMMIT;

    WHEN E_CAMPO_NULO THEN
      RSP := 'ERROR: EL CAMPO DEL DESARROLADOR EST EN BLANCO';

      INSERT INTO TFC.LOGS (datalog, PROCEDIMENT, PARAMSENTRADA, PARAMSSortida)
         VALUES(SYSDATE, 'SP_MODIF_DESENVOLUPADOR', param_entrada, 'RSP: '||RSP);

      COMMIT;
      
   
    WHEN OTHERS THEN
      err_num := SQLCODE;
      err_msg := SUBSTR(SQLERRM, 1, 200);

      RSP:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

      INSERT INTO tfc.LOGS (datalog, PROCEDIMENT, PARAMSENTRADA, paramssortida)
         VALUES(SYSDATE, 'SP_MODIF_DESENVOLUPADOR', param_entrada, 'RSP: '||RSP);

      COMMIT;


END SP_MODIF_DESENVOLUPADOR;
/
spool off;
