spool BD\16-logSP_BAJA_DESENVOLUPADOR.log

CONNECT TFC/hola;

create or replace PROCEDURE SP_BAJA_DESENVOLUPADOR (
   p_nomempresa 	IN  TFC.DESENVOLUPADORS.NOMEMPRESA%TYPE,
  
   
   RSP         		OUT VARCHAR2
)
IS
/******************************************************************************
   Nombre:                 SP_BAJA_DESENVOLUPADOR
   Propsito:            da de baja a un desarrollador a la tabla DESENVOLUPADORS
                   
   Parmetros entrada:   -	p_nomempresa 		VARCHAR2(20): 	Nombre de la empresa desarrolladora.
                                                               
   Parmetros salida:  RSP
   Retorna:             'OK' si la ejecucin ha finalizado con xito
                        'ERROR: tipo de error' si se ha producido un error
                        'ERROR' El Desarrollador no existe en la base de datos
******************************************************************************/

   E_CAMPO_NULO         EXCEPTION;
   E_DESENVOLUPADOR_EXISTE      EXCEPTION;

   existe        VARCHAR2(1);
   param_entrada VARCHAR2(500);
   err_num       NUMBER;
   err_msg       VARCHAR2(100);


   BEGIN

     param_entrada := 'P_NOMEMPRESA: '||p_nomempresa;

     --Verifica que todos los campos obligatorios tengan valor
     IF (TRIM(p_nomempresa)     IS NULL)
     THEN
         RAISE E_CAMPO_NULO;
      END IF;

     --Verifica que el registro no se haya insertado previmente con otro ID
     BEGIN

       SELECT 'S'
         INTO EXISTE
         FROM TFC.DESENVOLUPADORS
        WHERE NOMEMPRESA  = p_nomempresa
          AND ROWNUM  = 1;

        RAISE E_DESENVOLUPADOR_EXISTE;
      EXCEPTION
        WHEN NO_DATA_FOUND THEN
           NULL;
      END;

      --Si el desarrollador a dar de baja existe realiza el borrado

  
   
      RSP := 'ERROR: EL DESARROLLADOR NO EST DADO DE ALTA EN LA PLATAFORMA';

      INSERT INTO TFC.LOGS (datalog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
         VALUES(SYSDATE, 'SP_BAJA_DESENVOLUPADOR', param_entrada, 'RSP: '||RSP);
   

   

      INSERT INTO TFC.LOGS (datalog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
           VALUES(SYSDATE,'SP_BAJA_DESENVOLUPADOR', param_entrada, 'RSP: '||RSP);

      COMMIT;

  EXCEPTION
   WHEN E_DESENVOLUPADOR_EXISTE THEN
   DELETE FROM TFC.DESENVOLUPADORS 
        WHERE NOMEMPRESA  = p_nomempresa;

      RSP := 'OK';
      INSERT INTO TFC.LOGS (datalog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
           VALUES(SYSDATE,'SP_BAJA_DESENVOLUPADOR', param_entrada, 'RSP: '||RSP);
      
   
      COMMIT;

    WHEN E_CAMPO_NULO THEN
      RSP := 'ERROR: EL CAMPO DEL DESARROLLADOR EST EN BLANCO';

      INSERT INTO TFC.LOGS (datalog, PROCEDIMENT, PARAMSENTRADA, PARAMSSortida)
         VALUES(SYSDATE, 'SP_BAJA_DESENVOLUPADOR', param_entrada, 'RSP: '||RSP);

      COMMIT;

    WHEN OTHERS THEN
      err_num := SQLCODE;
      err_msg := SUBSTR(SQLERRM, 1, 200);

      RSP:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

      INSERT INTO tfc.LOGS (datalog, PROCEDIMENT, PARAMSENTRADA, paramssortida)
         VALUES(SYSDATE, 'SP_BAJA_DESENVOLUPADOR', param_entrada, 'RSP: '||RSP);

      COMMIT;


END SP_BAJA_DESENVOLUPADOR;

/
spool off;