CONNECT TFC/hola;
spool BD\17B-logSP_ALTA_DESENVOLUPAMENT.log
CREATE OR REPLACE PROCEDURE SP_ALTA_DESENVOLUPAMENT (
   p_nomempresa 	IN  tfc.desenvolupadors.nomempresa%TYPE,
    p_nomAplicacio 	    IN  tfc.aplicacions.nomaplicacio%TYPE,
   RSP         		OUT VARCHAR2
)
IS
/******************************************************************************
   Nombre:                 SP_ALTA_DESENVOLUPAMENT
   Propsito:            Agrega desarrolladores a aplicaciones desarrolladas por ms de un desarrollador
                   
   Parmetros entrada:   -	p_nomempresa 		VARCHAR2(20): 	Nombre de la empresa desarrolladora.
                         -  p_nomaplicacio  VARCHAR2(40):   Nombre de la aplicacin.
                                                               
   Parmetros salida:  RSP
   Retorna:             'OK' si la ejecucin ha finalizado con xito
                        'ERROR: tipo de error' si se ha producido un error
                        'ERROR' El Desarrollador no existe en la base de datos
                        'ERROR' La aplicacin no existe
******************************************************************************/

   E_CAMPO_NULO                    EXCEPTION;
   E_DESENVOLUPADOR_NO_EXISTE      EXCEPTION;
   E_APLICACION_NO_EXISTE          EXCEPTION;
   E_DESARROLLO_DUPLICADO          EXCEPTION;
   E_DESARROLLO_NO_EXISTE          EXCEPTION; 

   existe         VARCHAR2(1);
   existeapp      VARCHAR2(1);
   existedes      VARCHAR2(1);
   existedesenv   VARCHAR2(1);
   idaplic         VARCHAR2(40);
   iddesenv         VARCHAR2(20);
   param_entrada  VARCHAR2(500);
   err_num        NUMBER;
   err_msg        VARCHAR2(100);


   BEGIN

     param_entrada := 'P_NOMEMPRESA: '||p_nomempresa||',
     P_NOMAPLICACIO:'||p_nomaplicacio;

     --Verifica que todos los campos obligatorios tengan valor
     IF (TRIM(p_nomempresa)IS NULL) or (TRIM(p_nomaplicacio)IS NULL)
     THEN
         RAISE E_CAMPO_NULO;
      END IF;

     --Verifica que el desarrollador exista
     BEGIN

       SELECT 'S'
         INTO EXISTE
         FROM TFC.DESENVOLUPADORS
        WHERE desenvolupadors.NOMEMPRESA  = p_nomempresa
          AND ROWNUM  = 1;

        
      EXCEPTION
        WHEN NO_DATA_FOUND THEN
           RAISE E_DESENVOLUPADOR_NO_EXISTE;
      END;
    --Verifica que la aplicacin exista
      BEGIN

       SELECT 'S'
         INTO existeapp
         FROM TFC.aplicacions
        WHERE aplicacions.nomaplicacio  = p_nomaplicacio
          AND ROWNUM  = 1;

        
      EXCEPTION
        WHEN NO_DATA_FOUND THEN
           RAISE E_APLICACION_NO_EXISTE;
      END;
      
      
      BEGIN
    
        SELECT aplicacions.idaplicacio
        INTO idaplic
        FROM TFC.aplicacions
        WHERE aplicacions.nomaplicacio  = p_nomaplicacio
        AND ROWNUM  = 1;
          
        SELECT TFC.desenvolupadors.iddesenvolupador
        INTO iddesenv
        FROM TFC.desenvolupadors
        WHERE TFC.desenvolupadors.nomempresa  = p_nomempresa
        AND ROWNUM  = 1;

        SELECT 'S'
        INTO existedesenv
        FROM TFC.desenvolupaments
        WHERE TFC.desenvolupaments.iddesenvolupador  = iddesenv
        AND TFC.desenvolupaments.idaplicacio = idaplic
        AND ROWNUM  = 1;
        
        RSP := 'ERROR: EL DESARROLLO YA EXISTE';

        INSERT INTO TFC.LOGS (datalog, PROCEDIMENT, PARAMSENTRADA, PARAMSSortida)
        VALUES(SYSDATE, 'SP_ALTA_DESENVOLUPAMENT', param_entrada, 'RSP: '||RSP);
   
        EXCEPTION
        
        WHEN NO_DATA_FOUND THEN
        RAISE E_DESARROLLO_NO_EXISTE;
        
      END; 

COMMIT;

  EXCEPTION
   WHEN E_DESENVOLUPADOR_NO_EXISTE THEN
      RSP := 'ERROR: EL DESARROLLADOR NO EXISTE';

      INSERT INTO TFC.LOGS (datalog, PROCEDIMENT, PARAMSENTRADA, PARAMSSortida)
         VALUES(SYSDATE, 'SP_ALTA_DESENVOLUPAMENT', param_entrada, 'RSP: '||RSP);
   
   
      COMMIT;

  WHEN E_APLICACION_NO_EXISTE THEN
      RSP := 'ERROR: LA APLICACION NO EXISTE';

      INSERT INTO TFC.LOGS (datalog, PROCEDIMENT, PARAMSENTRADA, PARAMSSortida)
         VALUES(SYSDATE, 'SP_ALTA_DESENVOLUPAMENT', param_entrada, 'RSP: '||RSP);
   
   
      COMMIT;


WHEN E_DESARROLLO_NO_EXISTE THEN

      INSERT INTO TFC.DESENVOLUPAMENTS (iddesenvolupador,idaplicacio) VALUES (iddesenv,idaplic);
        RSP := 'OK';
        INSERT INTO TFC.LOGS (datalog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
        VALUES(SYSDATE,'SP_ALTA_DESENVOLUPAMENT', param_entrada, 'RSP: '||RSP);
        
        COMMIT;

    WHEN E_CAMPO_NULO THEN
      RSP := 'ERROR: EL CAMPO DEL DESARROLLADOR O DE LA APLICACIN ESTN EN BLANCO';

      INSERT INTO TFC.LOGS (datalog, PROCEDIMENT, PARAMSENTRADA, PARAMSSortida)
         VALUES(SYSDATE, 'SP_BAJA_DESENVOLUPADOR', param_entrada, 'RSP: '||RSP);

      COMMIT;

    WHEN OTHERS THEN
      err_num := SQLCODE;
      err_msg := SUBSTR(SQLERRM, 1, 200);

      RSP:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

      INSERT INTO tfc.LOGS (datalog, PROCEDIMENT, PARAMSENTRADA, paramssortida)
         VALUES(SYSDATE, 'SP_BAJA_DESENVOLUPADOR', param_entrada, 'RSP: '||RSP);

      COMMIT;


END SP_ALTA_DESENVOLUPAMENT;
/
SPOOL OFF;