spool BD\17-logSP_ALTA_APLICACIO.log

CONNECT TFC/hola;

create or replace PROCEDURE SP_ALTA_APLICACIO (
   p_nomAplicacio 	    IN  tfc.aplicacions.nomaplicacio%TYPE,
   p_versioAplicacio  	IN  tfc.aplicacions.versioaplicacio%TYPE,
   p_desenvolupador     IN  tfc.desenvolupadors.nomempresa%TYPE,
   p_urlvideo  	        IN  tfc.aplicacions.urlvideo%TYPE,
   p_resoluciMinima  	IN  tfc.aplicacions.resoluciminima%TYPE,
   p_resolucioMinima2  	IN  tfc.aplicacions.resoluciominima2%TYPE,
   p_esActiva  	        IN  tfc.aplicacions.esactiva%TYPE,
   p_idso               IN  tfc.sosuportats.idso%TYPE,
   p_urlAplicacio		    IN  tfc.sosuportats.urlaplicacio%TYPE,
   p_midaAplicacio      IN  tfc.sosuportats.midaaplicacio%TYPE,
   p_codPais            IN  tfc.preusaplicacions.codpais%TYPE,
   p_preuAplicacio      IN  tfc.preusaplicacions.preuaplicacio%TYPE,
   p_descripcio         IN  tfc.preusaplicacions.descripcio%TYPE,
   RSP         		OUT VARCHAR2
)
IS
/******************************************************************************
   Nombre:                 SP_ALTA_APLICACI
   Propsito:            Da de alta una aplicacin en el sistema. En la tabla
                   Aplicacions aade los datos que sern nicos para cada aplicacion
                   En la tabla sosuportats daremos de alta los SO soportados con
                   su tamao y direccin URL. En la tabla preusAplicacions ir el precio
                   para cada pis.
   Parmetros entrada: 
                   -   p_nomaplicacio    VARCHAR2(40) , Identificador de la aplicacin
                   -   p_versioAplicacio NUMBER , Versin de la aplicacin
                   -   p_desenvolupadors VARCHAR2(20), Nombre del desarrollador
                   -   p_urlvideo        VARCHAR2(100) , URL con una demo en video
                   -   p_resolucioMinima VARCHAR2(20), Resolucin mnima
                   -   p_esActiva        VARCHAR2(2) , La aplicacin est activa(SI/NO)
                  -    p_idso   		     NUMBER , cdigo del S.O. de la aplicacin
                  -    p_urlAplicacio    VARCHAR2(200), Descarga de la aplicacin
                  -    p_midaAplicacio   NUMBER, tamao de la aplicacin en Bytes
                  -    p_codPais         VARCHAR2(2), Cdigo del Pas donde est disponible
                  -    p_preuAplicacio   NUMBER , precio de la aplicacin en ese Pais
                  -    p_descripcio      VARCHAR2(100), Descripcin de la aplicacin
                  
   Parmetros salida:  RSP
   Retorna:             'OK' si la ejecucin ha finalizado con xito
                        'ERROR: tipo de error' si se ha producido un error
******************************************************************************/

   E_CAMPO_NULO                 EXCEPTION;
   E_PAIS_NO_EXISTE             EXCEPTION;
   E_SO_NO_EXISTE               EXCEPTION;
   E_APLICACION_EXISTE          EXCEPTION;
   E_DESARROLLADOR_NO_EXISTE    EXCEPTION;
   idAplic                      NUMBER;
   idDesenv                     NUMBER;
   existe                       VARCHAR2(1);
   existepais                   VARCHAR2(1);
   existeso                     VARCHAR2(1);
   existedesarrollador          VARCHAR2(1);
   fechaalta                    DATE;
   param_entrada                VARCHAR2(500);
   err_num                      NUMBER;
   err_msg                      VARCHAR2(100);
  

   BEGIN

     param_entrada := 'P_NOMAPLICACIO:'||p_nomaplicacio||',
     P_VERSIOAPLICACIO:'||p_versioAplicacio||',
     P_DESENVOLUPADOR:'||p_desenvolupador||',
     P_URLVIDEO:'||p_urlvideo||',
     P_RESOLUCIMINIMA:'||p_resoluciminima||',
     P_RESOLUCIOMINIMA2:'||p_resoluciominima2||',
     P_ESACTIVA:'||p_esActiva||',
     P_IDSO:'||p_idso||',
     P_URLAPLICACIO:'||p_urlAplicacio||',
     P_MIDAAPLICACIO:'||p_midaAplicacio||',
     P_CODPAIS:'||p_codPais||',
     P_PREUAPLICACI:'||p_preuAplicacio||',
     P_DESCRIPCIO:'||p_descripcio;
     
     

     --Verifica que todos los campos obligatorios tengan valor
     IF (TRIM(p_nomaplicacio) IS NULL OR TRIM(p_versioAPlicacio) IS NULL
     OR TRIM(p_desenvolupador) IS NULL
     OR TRIM(p_resoluciMinima) IS NULL OR TRIM(p_versioAplicacio)IS NULL
     OR TRIM(p_esActiva) IS NULL OR TRIM(p_idso)IS NULL
     OR TRIM(p_urlAplicacio) IS NULL OR TRIM(p_midaAplicacio)IS NULL
     OR TRIM(p_codPais) IS NULL OR TRIM(p_preuAplicacio)IS NULL
     OR TRIM(p_descripcio) IS NULL)
     THEN
         RAISE E_CAMPO_NULO;
      END IF;

     --Verifica que el registro no se haya insertado previmente con otro ID
     BEGIN

       SELECT 'S'
         INTO EXISTE
         FROM tfc.aplicacions
        WHERE nomaplicacio  = p_nomAplicacio
          AND ROWNUM  = 1;

        RAISE E_APLICACION_EXISTE;
      EXCEPTION
        WHEN NO_DATA_FOUND THEN
           NULL;
      END;
--Verifica que el primer pas y su descripcin asociada para el que se da de alta la aplicacin existe

       BEGIN
       SELECT 'S'
         INTO existepais
         FROM tfc.paisos
        WHERE CODPAIS  = p_codpais
          AND ROWNUM  = 1;
        EXCEPTION
        WHEN NO_DATA_FOUND THEN
        RAISE  E_PAIS_NO_EXISTE;
      END;
      
--Verifica que la aplicacin corre bajo un SO registrado en el sistema
      
       BEGIN
       SELECT 'S'
         INTO existeso
         FROM tfc.sistemasoperatius
        WHERE idso  = p_idso
          AND ROWNUM  = 1;
        EXCEPTION
        WHEN NO_DATA_FOUND THEN
        RAISE  E_SO_NO_EXISTE;
      END;
      
  --Verifica que el desarrollador existe   
       
       BEGIN
       SELECT 'S'
         INTO existedesarrollador
         FROM tfc.desenvolupadors
        WHERE NOMEMPRESA  = p_desenvolupador
          AND ROWNUM  = 1;
        EXCEPTION
        WHEN NO_DATA_FOUND THEN
        RAISE  E_DESARROLLADOR_NO_EXISTE;
      END;
      
      --Si ha pasado todas las pruebas de validacin realiza la insercin
      
      SELECT systimestamp 
     INTO fechaalta
      FROM SYS.dual;
      
      INSERT INTO tfc.aplicacions (nomaplicacio,versioaplicacio,urlvideo,resoluciminima,resoluciominima2,esactiva,datapujada)
           VALUES (p_nomAplicacio, p_versioAplicacio,p_urlvideo,p_resoluciMinima,p_resolucioMinima2,p_esActiva,fechaalta);
      
      
      
     -- INTO fechaalta
      
      
     
      
      SELECT idaplicacio
      INTO idaplic
      FROM tfc.aplicacions
      WHERE nomaplicacio = p_nomAplicacio;
      
      INSERT INTO tfc.sosuportats (idaplicacio,idso,urlaplicacio,midaaplicacio)
           VALUES (idaplic, p_idso, p_urlaplicacio,p_midaaplicacio);

      INSERT INTO tfc.preusaplicacions (idaplicacio,codpais,preuAplicacio,descripcio)
           VALUES (idaplic, p_codpais, p_preuAplicacio,p_descripcio);

      SELECT iddesenvolupador
      INTO idDesenv
      FROM tfc.desenvolupadors
      WHERE nomempresa = p_desenvolupador;

      INSERT INTO tfc.desenvolupaments (idDesenvolupador,idAplicacio)
           VALUES (idDesenv,idAplic);


      RSP := 'OK';

      INSERT INTO TFC.LOGS (datalog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
           VALUES(SYSDATE,'SP_ALTA_APLICACIO', param_entrada, 'RSP: '||RSP);

      COMMIT;

  EXCEPTION
    WHEN E_APLICACION_EXISTE THEN
      RSP := 'ERROR: LA APLICACIN YA EST DADA DE ALTA EN LA PLATAFORMA';

      INSERT INTO TFC.LOGS (datalog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
         VALUES(SYSDATE, 'SP_ALTA_APLICACIO', param_entrada, 'RSP: '||RSP);

      COMMIT;

    WHEN E_CAMPO_NULO THEN
      RSP := 'ERROR: ALGUNO DE LOS CAMPOS OBLIGATORIOS EST VACO';

      INSERT INTO TFC.LOGS (datalog, PROCEDIMENT, PARAMSENTRADA, PARAMSSortida)
         VALUES(SYSDATE, 'SP_ALTA_APLICACIO', param_entrada, 'RSP: '||RSP);

      COMMIT;

    WHEN E_PAIS_NO_EXISTE THEN
      RSP := 'ERROR: EL PAS NO EXISTE';

      INSERT INTO LOGS (DATALOG, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
         VALUES(SYSDATE, 'SP_ALTA_APLICACIO', param_entrada, 'RSP: '||RSP);

      COMMIT;
      
  WHEN E_SO_NO_EXISTE THEN
      RSP := 'ERROR: EL SO NO EST DADO DE ALTA EN LA PLATAFORMA';

      INSERT INTO LOGS (DATALOG, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
         VALUES(SYSDATE, 'SP_ALTA_APLICACIO', param_entrada, 'RSP: '||RSP);

      COMMIT;
      
  WHEN E_DESARROLLADOR_NO_EXISTE THEN
      RSP := 'ERROR: EL DESARROLLADOR NO EXISTE';

      INSERT INTO LOGS (DATALOG, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
         VALUES(SYSDATE, 'SP_ALTA_APLICACIO', param_entrada, 'RSP: '||RSP);

      COMMIT;


    WHEN OTHERS THEN
      err_num := SQLCODE;
      err_msg := SUBSTR(SQLERRM, 1, 200);

      RSP:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

      INSERT INTO tfc.LOGS (datalog, PROCEDIMENT, PARAMSENTRADA, paramssortida)
         VALUES(SYSDATE, 'SP_ALTA_APLICACIO', param_entrada, 'RSP: '||RSP);

      COMMIT;


END SP_ALTA_APLICACIO;
/
spool off;