SPOOL BD\18-logSP_ANADIR_SO.log

CONNECT TFC/hola;

create or replace PROCEDURE SP_ANADIR_SO (
   p_nomAplicacio 	    IN  tfc.aplicacions.nomaplicacio%TYPE,
   p_idso               IN  tfc.sosuportats.idso%TYPE,
   p_urlAplicacio		    IN  tfc.sosuportats.urlaplicacio%TYPE,
   p_midaAplicacio      IN  tfc.sosuportats.midaaplicacio%TYPE,
   RSP         		OUT VARCHAR2
)
IS
/******************************************************************************
   Nombre:                 SP_ANADIR_SO_APLICACI
   Propsito:            Aade un sistema operativo soportado para una aplicacion
                        existente. Introduce en el sistema los datos de SO, url de la
                        aplicacin y el tamao de la versin para ese S.O.
   Parmetros entrada: 
                  -   p_nomaplicacio    VARCHAR2(40) , Identificador de la aplicacin
                  -    p_idso   		     NUMBER , cdigo del S.O. de la aplicacin
                  -    p_urlAplicacio    VARCHAR2(200), Descarga de la aplicacin
                  -    p_midaAplicacio   NUMBER, tamao de la aplicacin en Bytes
                  
   Parmetros salida:  RSP
   Retorna:             'OK' si la ejecucin ha finalizado con xito
                        'ERROR: tipo de error' si se ha producido un error
******************************************************************************/

   E_CAMPO_NULO           EXCEPTION;
   E_SO_NO_EXISTE          EXCEPTION;
   E_APLICACION_NO_EXISTE  EXCEPTION;
   E_SO_YA_SOPORTADO         EXCEPTION;
   idAplic              NUMBER;
   existe               VARCHAR2(1);
   existepais           VARCHAR2(1);
   existeso           VARCHAR2(1);
   existesoporte      VARCHAR2(1);
   param_entrada        VARCHAR2(500);
   err_num              NUMBER;
   err_msg              VARCHAR2(100);
  

   BEGIN

     param_entrada := 'P_NOMAPLICACIO:'||p_nomaplicacio||',
     P_IDSO:'||p_idso||',
     P_URLAPLICACIO:'||p_urlAplicacio||',
     P_MIDAAPLICACIO:'||p_midaAplicacio;
     
     

     --Verifica que todos los campos obligatorios tengan valor
     IF (TRIM(p_nomaplicacio) IS NULL OR TRIM(p_idso)IS NULL
     OR TRIM(p_urlAplicacio) IS NULL OR TRIM(p_midaAplicacio)IS NULL)
     THEN
         RAISE E_CAMPO_NULO;
      END IF;

     --Verifica que la aplicacin a la cual queremos aadir un SO existe
     BEGIN

       SELECT 'S'
         INTO EXISTE
         FROM tfc.aplicacions
        WHERE nomaplicacio  = p_nomAplicacio
          AND ROWNUM  = 1;

      EXCEPTION
        WHEN NO_DATA_FOUND THEN
        RAISE E_APLICACION_NO_EXISTE;
           
      END;
--Verifica que la aplicacin corre bajo un SO registrado en el sistema
      
       BEGIN
       SELECT 'S'
         INTO existeso
         FROM tfc.sistemasoperatius
        WHERE idso  = p_idso
          AND ROWNUM  = 1;
        EXCEPTION
        WHEN NO_DATA_FOUND THEN
        RAISE  E_SO_NO_EXISTE;
      END;
      
 --Verifica que la aplicacin  no corra ya bajo un SO registrado en el sistema     
      
      
      SELECT idaplicacio
      INTO idaplic
      FROM tfc.aplicacions
      WHERE nomaplicacio = p_nomAplicacio;
      
         BEGIN
       SELECT 'S'
         INTO existesoporte
         FROM tfc.sosuportats
        WHERE sosuportats.idso  = p_idso
        AND sosuportats.idaplicacio = idaplic
          AND ROWNUM  = 1;
        RAISE  E_SO_YA_SOPORTADO;
        EXCEPTION
        WHEN NO_DATA_FOUND THEN
        NULL;
      END;
      
      
      --Si ha pasado todas las pruebas de validacin realiza la insercin
      
     
      
      
      INSERT INTO tfc.sosuportats (idaplicacio,idso,urlaplicacio,midaaplicacio)
           VALUES (idaplic, p_idso, p_urlaplicacio,p_midaaplicacio);

      RSP := 'OK';

      INSERT INTO TFC.LOGS (datalog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
           VALUES(SYSDATE,'SP_ANADIR_SO', param_entrada, 'RSP: '||RSP);

      COMMIT;

  EXCEPTION
    WHEN E_APLICACION_NO_EXISTE THEN
      RSP := 'ERROR: LA APLICACIN NO EST DE ALTA EN LA PLATAFORMA';

      INSERT INTO TFC.LOGS (datalog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
         VALUES(SYSDATE, 'SP_ANADIR_SO', param_entrada, 'RSP: '||RSP);

      COMMIT;

    WHEN E_CAMPO_NULO THEN
      RSP := 'ERROR: ALGUNO DE LOS CAMPOS OBLIGATORIOS ES NULO';

      INSERT INTO TFC.LOGS (datalog, PROCEDIMENT, PARAMSENTRADA, PARAMSSortida)
         VALUES(SYSDATE, 'SP_ANADIR_SO', param_entrada, 'RSP: '||RSP);

      COMMIT;

  WHEN E_SO_NO_EXISTE THEN
      RSP := 'ERROR: EL SO NO EST SOPORTADO EN LA PLATAFORMA';

      INSERT INTO LOGS (DATALOG, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
         VALUES(SYSDATE, 'SP_ANADIR_SO', param_entrada, 'RSP: '||RSP);

      COMMIT;


    WHEN E_SO_YA_SOPORTADO THEN
      RSP := 'ERROR: LA APLICACIN YA EST SOPORTADO POR ESE S.O.';

      INSERT INTO TFC.LOGS (datalog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
         VALUES(SYSDATE, 'SP_ANADIR_SO', param_entrada, 'RSP: '||RSP);

      COMMIT;



    WHEN OTHERS THEN
      err_num := SQLCODE;
      err_msg := SUBSTR(SQLERRM, 1, 200);

      RSP:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

      INSERT INTO tfc.LOGS (datalog, PROCEDIMENT, PARAMSENTRADA, paramssortida)
         VALUES(SYSDATE, 'SP_ANADIR_SO', param_entrada, 'RSP: '||RSP);

      COMMIT;


END SP_ANADIR_SO;
/
spool off;