spool BD\19-logSP_ANADIR_PRECIO.log

CONNECT TFC/hola;

create or replace PROCEDURE SP_ANADIR_PRECIO (
   p_nomAplicacio 	    IN  tfc.aplicacions.nomaplicacio%TYPE,
   p_codPais            IN  tfc.preusaplicacions.codpais%TYPE,
   p_preuAplicacio      IN  tfc.preusaplicacions.preuaplicacio%TYPE,
   p_descripcio         IN  tfc.preusaplicacions.descripcio%TYPE,
   RSP         		OUT VARCHAR2
)
IS
/******************************************************************************
   Nombre:                 SP_ANADIR_PRECIO
   Propsito:            Permite aadir paises para los que la aplicacin estar
                      Disponible con sus precios y descripciones correspondientes
   Parmetros entrada: 
                   -   p_nomaplicacio    VARCHAR2(40) , Identificador de la aplicacin
                   -    p_codPais         VARCHAR2(2), Cdigo del Pas donde est disponible
                  -    p_preuAplicacio   NUMBER , precio de la aplicacin en ese Pais
                  -    p_descripcio      VARCHAR2(100), Descripcin de la aplicacin
                  
   Parmetros salida:  RSP
   Retorna:             'OK' si la ejecucin ha finalizado con xito
                        'ERROR: tipo de error' si se ha producido un error
******************************************************************************/

   E_CAMPO_NULO            EXCEPTION;
   E_PAIS_NO_EXISTE        EXCEPTION;
   
   E_APLICACION_NO_EXISTE  EXCEPTION;
   E_PRECIO_EXISTE         EXCEPTION;
   idAplic                 NUMBER;
   existe                  VARCHAR2(1);
   existepais              VARCHAR2(1);
   existeso                VARCHAR2(1);
   existeprecio            VARCHAR2(1);
   fechaalta               DATE;
   param_entrada          VARCHAR2(500);
   err_num                NUMBER;
   err_msg                VARCHAR2(100);
  

   BEGIN

     param_entrada := 'P_NOMAPLICACIO:'||p_nomaplicacio||',
     P_CODPAIS:'||p_codPais||',
     P_PREUAPLICACI:'||p_preuAplicacio||',
     P_DESCRIPCIO:'||p_descripcio;
     
     
     --Verifica que todos los campos obligatorios tengan valor
     IF (TRIM(p_nomaplicacio) IS NULL 
     OR TRIM(p_codPais) IS NULL 
     OR TRIM(p_preuAplicacio)IS NULL
     OR TRIM(p_descripcio) IS NULL)
     THEN
         RAISE E_CAMPO_NULO;
      END IF;

     --Verifica que la aplicacin exista
     BEGIN

       SELECT 'S'
         INTO EXISTE
         FROM tfc.aplicacions
        WHERE nomaplicacio  = p_nomAplicacio
          AND ROWNUM  = 1;

        
      EXCEPTION
        WHEN NO_DATA_FOUND THEN
        RAISE E_APLICACION_NO_EXISTE;
      END;
--Verifica que el pas y su descripcin asociada para el que se da de alta la aplicacin existe

       BEGIN
       SELECT 'S'
         INTO existepais
         FROM tfc.paisos
        WHERE CODPAIS  = p_codpais
          AND ROWNUM  = 1;
        EXCEPTION
        WHEN NO_DATA_FOUND THEN
        RAISE  E_PAIS_NO_EXISTE;
      END;
      
      --Si ha pasado todas las pruebas de validacin realiza la insercin
      
      
      SELECT idaplicacio
      INTO idaplic
      FROM tfc.aplicacions
      WHERE nomaplicacio = p_nomAplicacio;
      
       BEGIN
       SELECT 'S'
         INTO existeprecio
         FROM tfc.preusaplicacions
        WHERE CODPAIS  = p_codpais
        AND preusaplicacions.idaplicacio =idaplic
          AND ROWNUM  = 1;
          RAISE  E_PRECIO_EXISTE;
        EXCEPTION
        WHEN NO_DATA_FOUND THEN
        NULL;
      END;
      
      
      INSERT INTO tfc.preusaplicacions (idaplicacio,codpais,preuaplicacio,descripcio)
           VALUES (idaplic, p_codpais, p_preuaplicacio,p_descripcio);

      

      RSP := 'OK';

      INSERT INTO TFC.LOGS (datalog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
           VALUES(SYSDATE,'SP_ANADIR_PRECIO', param_entrada, 'RSP: '||RSP);

      COMMIT;

  EXCEPTION
    WHEN E_APLICACION_NO_EXISTE THEN
      RSP := 'ERROR: LA APLICACIN NO EST DE ALTA EN EL SISTEMA';

      INSERT INTO TFC.LOGS (datalog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
         VALUES(SYSDATE, 'SP_ANADIR_PRECIO', param_entrada, 'RSP: '||RSP);

      COMMIT;

    WHEN E_CAMPO_NULO THEN
      RSP := 'ERROR: ALGUNO DE LOS CAMPOS OBLIGATORIOS ES NULO';

      INSERT INTO TFC.LOGS (datalog, PROCEDIMENT, PARAMSENTRADA, PARAMSSortida)
         VALUES(SYSDATE, 'SP_ANADIR_PRECIO', param_entrada, 'RSP: '||RSP);

      COMMIT;

    WHEN E_PAIS_NO_EXISTE THEN
      RSP := 'ERROR: EL PAS NO EXISTE';

      INSERT INTO LOGS (DATALOG, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
         VALUES(SYSDATE, 'SP_ANADIR_PRECIO', param_entrada, 'RSP: '||RSP);

      COMMIT;
  
  
    WHEN E_PRECIO_EXISTE THEN
      RSP := 'ERROR: YA HAY PRECIO ASIGNADO PARA ESE PAS';

      INSERT INTO LOGS (DATALOG, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
         VALUES(SYSDATE, 'SP_ANADIR_PRECIO', param_entrada, 'RSP: '||RSP);

      COMMIT;
  
    WHEN OTHERS THEN
      err_num := SQLCODE;
      err_msg := SUBSTR(SQLERRM, 1, 200);

      RSP:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

      INSERT INTO tfc.LOGS (datalog, PROCEDIMENT, PARAMSENTRADA, paramssortida)
         VALUES(SYSDATE, 'SP_MODIF_PRECIO', param_entrada, 'RSP: '||RSP);

      COMMIT;


END SP_ANADIR_PRECIO;
/
spool off;