spool BD\21-logSP_MODIF_APLICACIO.log

CONNECT TFC/hola;

create or replace PROCEDURE SP_MODIF_APLICACIO(
   p_nomaplicacio       IN  tfc.aplicacions.nomaplicacio%TYPE,   
   p_urlvideo  	        IN  tfc.aplicacions.urlvideo%TYPE,
   p_resoluciMinima  	IN  tfc.aplicacions.resoluciminima%TYPE,
   p_resolucioMinima2  	IN  tfc.aplicacions.resoluciominima2%TYPE,
   RSP         		OUT VARCHAR2
)
IS
/******************************************************************************
   Nombre:                 SP_MODIF_APLICACIO
   Propsito:            Permite modificar datos no esenciales de una aplicacin como pueden ser
                         la direccin con el video o la resolucin. 
   Parmetros entrada: 
                   -   p_nomaplicacio    VARCHAR2(40) , Identificador de la aplicacin
                   -   p_urlvideo        VARCHAR2(100) , URL con una demo en video
                   -   p_resolucioMinima VARCHAR2(20), Resolucin mnima Ancho
                   -   p_resolucioMinima2 VARCHAR2(20), Resolucin mnima Alto
                  
   Parmetros salida:  RSP
   Retorna:             'OK' si la ejecucin ha finalizado con xito
                        'ERROR: tipo de error' si se ha producido un error
******************************************************************************/

   E_CAMPO_NULO                 EXCEPTION;
   E_APLICACION_NO_EXISTE       EXCEPTION;
   idAplic                      NUMBER;
   existe                       VARCHAR2(1);
   param_entrada                VARCHAR2(500);
   err_num                      NUMBER;
   err_msg                      VARCHAR2(100);
  

   BEGIN

     param_entrada := 'P_NOMAPLICACIO:'||p_nomaplicacio||',
     P_URLVIDEO:'||p_urlvideo||',
     P_RESOLUCIMINIMA:'||p_resoluciminima||',
     P_RESOLUCIOMINIMA2:'||p_resoluciominima2;
     

     --Verifica que todos los campos obligatorios tengan valor
     IF (TRIM(p_nomaplicacio) IS NULL 
     OR TRIM(p_resoluciMinima) IS NULL OR TRIM(p_resolucioMinima2)IS NULL )
     
     THEN
         RAISE E_CAMPO_NULO;
      END IF;

     --Verifica que la aplicacin a modificar exista
     BEGIN

       SELECT 'S'
         INTO EXISTE
         FROM tfc.aplicacions
        WHERE nomaplicacio  = p_nomAplicacio
          AND ROWNUM  = 1;
      EXCEPTION
      WHEN NO_DATA_FOUND THEN
      RAISE E_APLICACION_NO_EXISTE;
           
      END;
      
      --Si ha pasado todas las pruebas de validacin modifica los datos
      
      
     
      UPDATE tfc.aplicacions
      SET tfc.aplicacions.urlvideo = nvl(p_urlvideo,urlvideo),
      tfc.aplicacions.resoluciminima = nvl(p_resoluciMinima,resoluciMinima),
      tfc.aplicacions.resoluciominima2 = nvl(p_resolucioMinima2,resolucioMinima2)
      WHERE tfc.aplicacions.nomaplicacio = p_nomaplicacio;
      
      RSP := 'OK';

      INSERT INTO TFC.LOGS (datalog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
           VALUES(SYSDATE,'SP_MODIF_APLICACIO', param_entrada, 'RSP: '||RSP);

      COMMIT;

  EXCEPTION
    WHEN E_APLICACION_NO_EXISTE THEN
      RSP := 'ERROR: LA APLICACIN NO EST DE ALTA EN LA PLATAFORMA';

      INSERT INTO TFC.LOGS (datalog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
         VALUES(SYSDATE, 'SP_MODIF_APLICACIO', param_entrada, 'RSP: '||RSP);

      COMMIT;

    WHEN E_CAMPO_NULO THEN
      RSP := 'ERROR: ALGUNO DE LOS CAMPOS OBLIGATORIOS ES NULO';

      INSERT INTO TFC.LOGS (datalog, PROCEDIMENT, PARAMSENTRADA, PARAMSSortida)
         VALUES(SYSDATE, 'SP_MODIF_APLICACIO', param_entrada, 'RSP: '||RSP);

      COMMIT;

    WHEN OTHERS THEN
      err_num := SQLCODE;
      err_msg := SUBSTR(SQLERRM, 1, 200);

      RSP:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

      INSERT INTO tfc.LOGS (datalog, PROCEDIMENT, PARAMSENTRADA, paramssortida)
         VALUES(SYSDATE, 'SP_MODIF_APLICACIO', param_entrada, 'RSP: '||RSP);

      COMMIT;


END SP_MODIF_APLICACIO;
/
spool off;