spool BD\22-logSP_ACTIVAR.log

CONNECT TFC/hola;

create or replace PROCEDURE SP_ACTIVAR(
   p_nomaplicacio       IN  tfc.aplicacions.nomaplicacio%TYPE,   
   p_esActiva           IN  tfc.aplicacions.esActiva%TYPE,
   RSP         		OUT VARCHAR2
)
IS
/******************************************************************************
   Nombre:                 SP_ACTIVAR
   Propsito:            Cambia el estado de la aplicacin (Activa o no mediante un SI o NO)
   Parmetros entrada: 
                   -   p_nomaplicacio    VARCHAR2(40) , Identificador de la aplicacin
                   -   p_esActiva        VARCHAR2(2) , Identificador de la aplicacin
   Parmetros salida:  RSP
   Retorna:             'OK' si la ejecucin ha finalizado con xito
                        'ERROR: tipo de error' si se ha producido un error
******************************************************************************/

   E_CAMPO_NULO                 EXCEPTION;
   E_APLICACION_NO_EXISTE       EXCEPTION;
   idAplic                      NUMBER;
   existe                       VARCHAR2(1);
   param_entrada                VARCHAR2(500);
   err_num                      NUMBER;
   err_msg                      VARCHAR2(100);
  

   BEGIN

     param_entrada := 'P_NOMAPLICACIO:'||p_nomaplicacio||',
     P_ESACTIVA:'||p_esActiva;

     --Verifica que todos los campos obligatorios tengan valor
     IF (TRIM(p_nomaplicacio) IS NULL 
     OR TRIM(p_esActiva) IS NULL )
     
     THEN
         RAISE E_CAMPO_NULO;
      END IF;

     --Verifica que la aplicacin a activar exista
     BEGIN

       SELECT 'S'
         INTO EXISTE
         FROM tfc.aplicacions
        WHERE nomaplicacio  = p_nomAplicacio
          AND ROWNUM  = 1;
      EXCEPTION
      WHEN NO_DATA_FOUND THEN
      RAISE E_APLICACION_NO_EXISTE;
           
      END;
      
      --Si ha pasado todas las pruebas de validacin modifica los datos
      
      
     
      UPDATE tfc.aplicacions
      SET tfc.aplicacions.esActiva = nvl(p_esActiva,esActiva)
      WHERE tfc.aplicacions.nomaplicacio = p_nomaplicacio;
      
      RSP := 'OK';

      INSERT INTO TFC.LOGS (datalog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
           VALUES(SYSDATE,'SP_ACTIVAR', param_entrada, 'RSP: '||RSP);

      COMMIT;

  EXCEPTION
    WHEN E_APLICACION_NO_EXISTE THEN
      RSP := 'ERROR: LA APLICACIN NO EST DADA DE ALTA EN EL SISTEMA';

      INSERT INTO TFC.LOGS (datalog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
         VALUES(SYSDATE, 'SP_ACTIVAR', param_entrada, 'RSP: '||RSP);

      COMMIT;

    WHEN E_CAMPO_NULO THEN
      RSP := 'ERROR: UN CAMPO OBLIGATORIO EST VACO';

      INSERT INTO TFC.LOGS (datalog, PROCEDIMENT, PARAMSENTRADA, PARAMSSortida)
         VALUES(SYSDATE, 'SP_ACTIVAR', param_entrada, 'RSP: '||RSP);

      COMMIT;

    WHEN OTHERS THEN
      err_num := SQLCODE;
      err_msg := SUBSTR(SQLERRM, 1, 200);

      RSP:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

      INSERT INTO tfc.LOGS (datalog, PROCEDIMENT, PARAMSENTRADA, paramssortida)
         VALUES(SYSDATE, 'SP_ACTIVAR', param_entrada, 'RSP: '||RSP);

      COMMIT;


END SP_ACTIVAR;
/
spool off;