

spool BD\23-logSP_DESCARGAR_APLICACIO.log

CONNECT TFC/hola;

create or replace PROCEDURE SP_DESCARGAR_APLICACIO (
 p_nomAplicacio 	    IN  tfc.aplicacions.nomaplicacio%TYPE,
 p_codimei            IN  tfc.imei.codimei%TYPE,
 p_modepagament       IN  tfc.descarregas.modepagament%TYPE,
 p_diadesc            IN  tfc.descarregas.diadesc%TYPE,
 p_mesdesc            IN  tfc.descarregas.mesdesc%TYPE,
 p_anodesc            IN  tfc.descarregas.anodesc%TYPE,
 RSP                  OUT VARCHAR2
 )
 IS
/******************************************************************************
   Nombre:                 SP_DESCARGAR_APLICACIN
   Propsito:            Guarda los datos asociados a la descarga de una aplicacin a un 
                        dispositivo de un usuario vlido. Para ello la se encesita introducir 
                        un nombre de aplicacin un imei vlidos
   Parmetros entrada: 
                   -   p_nomaplicacio    VARCHAR2(40) , Identificador de la aplicacin
                   -   p_codimei         VARCHAR2(2), Cdigo del Imei donde se va a descargar
                   -   p_modepagament    VARCHAR2(10), Modo de pago 
   Parmetros salida:  RSP
   Retorna:             'OK' si la ejecucin ha finalizado con xito
                        'ERROR: tipo de error' si se ha producido un error
******************************************************************************/ 

   E_CAMPO_NULO               EXCEPTION;
   E_APLICACION_NO_EXISTE     EXCEPTION;
   E_IMEI_NO_EXISTE           EXCEPTION;
   E_USER_NO_EXISTE           EXCEPTION;
   E_PREU_NO_EXISTE           EXCEPTION;
   E_PAIS_NO_EXISTE           EXCEPTION;
   E_SO_NO_SOPORTADO          EXCEPTION; 
   E_RESOLUCION_NO_SOPORTADA  EXCEPTION;
   E_FECHA_DESCARGA_INVALIDA  EXCEPTION;
   idAplic              NUMBER;
   idUser               NUMBER;
   soimei               NUMBER;
   soAPlic              NUMBER;
   rd1                  NUMBER;
   rd2                  NUMBER;
   rd1OK                NUMBER;
   rd2OK                NUMBER;
   paisUser             VARCHAR2(2);
   preuDescarrega       NUMBER;
   factDescarregas      NUMBER;
   nDescarregas         NUMBER;
   existeAplicacio      VARCHAR2(1);
   existeImei           VARCHAR2(1);
   existeUser           VARCHAR2(1);
   existePreu           VARCHAR2(1);
   aodescarrega        NUMBER;
   n_diadesc           NUMBER;
   n_mesdesc           NUMBER;
   n_anodesc           NUMBER;
   data_entrada         VARCHAR2(8);
   date_data_pujada     DATE;
   param_entrada        VARCHAR2(500);
   err_num              NUMBER;
   err_msg              VARCHAR2(100);
   
   BEGIN

     param_entrada := 'P_NOMAPLICACIO:'||p_nomaplicacio||',
     P_CODIMEI:'||p_codimei||',
     P_MODOPAGAMENT:'||p_modepagament;
     
     
     --Verifica que todos los campos obligatorios tengan valor
     IF (TRIM(p_nomaplicacio) IS NULL 
     OR TRIM(p_codimei) IS NULL
     OR TRIM(p_modepagament) IS NULL)
     THEN
         RAISE E_CAMPO_NULO;
      END IF;
      
      --Verifica que la aplicacin exista y est activa
     BEGIN

       SELECT 'S'
         INTO existeaplicacio
         FROM tfc.aplicacions
        WHERE nomaplicacio  = p_nomAplicacio
        AND (tfc.aplicacions.esactiva = 'si' OR tfc.aplicacions.esactiva = 'SI')
          AND ROWNUM  = 1;

        
      EXCEPTION
        WHEN NO_DATA_FOUND THEN
        RAISE E_APLICACION_NO_EXISTE;
      END;
      
      --Verifica que el imei existe y est asociado a un usuario
     BEGIN

       SELECT 'S'
         INTO existeImei
         FROM tfc.imei
        WHERE codimei  = p_codimei
        --AND tfc.imei.idusuariimei IS NOT NULL
          AND ROWNUM  = 1;

        
      EXCEPTION
        WHEN NO_DATA_FOUND THEN
        RAISE E_IMEI_NO_EXISTE;
      END;
      
      --Toma los datos de precio, usuario y aplicacin para hacer la insercin
      BEGIN
      SELECT idaplicacio
      INTO idaplic
      FROM tfc.aplicacions
      WHERE nomaplicacio = p_nomAplicacio
      AND ROWNUM  = 1;
      
      EXCEPTION
        WHEN NO_DATA_FOUND THEN
        RAISE E_APLICACION_NO_EXISTE;
      END;
      
      BEGIN
      SELECT tfc.imei.idusuariimei
      INTO idUser
      FROM tfc.imei
      WHERE codimei = p_codimei;
      
      EXCEPTION
        WHEN NO_DATA_FOUND THEN
        RAISE E_USER_NO_EXISTE;
      END;
      
      BEGIN

       SELECT 'S'
         INTO existeUser
         FROM tfc.imei
        WHERE codimei  = p_codimei
        AND tfc.imei.idusuariimei =idUser
          AND ROWNUM  = 1;

        
      EXCEPTION
        WHEN NO_DATA_FOUND THEN
        RAISE E_USER_NO_EXISTE;
      END;
      
      
      BEGIN
      SELECT paisusuari
      INTO paisUser
      FROM tfc.usuaris
      WHERE tfc.usuaris.idusuari = idUser;
      EXCEPTION
        WHEN NO_DATA_FOUND THEN
        RAISE E_PAIS_NO_EXISTE;
      END;
      
      BEGIN 
      
      
      SELECT preuaplicacio
      into preuDescarrega
      from tfc.preusaplicacions
      
      WHERE paisUser = tfc.preusaplicacions.codpais 
      AND idAplic = tfc.preusaplicacions.idAplicacio
       AND ROWNUM  = 1;
   
     
      EXCEPTION
        WHEN NO_DATA_FOUND THEN
        RAISE E_PREU_NO_EXISTE;
      END;
   
      BEGIN

       SELECT 'S'
         INTO existePreu
         FROM tfc.preusAPlicacions
       WHERE paisUser = tfc.preusaplicacions.codpais 
      AND idAplic = tfc.preusaplicacions.idAplicacio
          AND ROWNUM  = 1;

        
      EXCEPTION
        WHEN NO_DATA_FOUND THEN
        RAISE E_PREU_NO_EXISTE;
      END;
   
   BEGIN
   SELECT tfc.imei.sodispositiu
   INTO soimei
   FROM tfc.imei
   WHERE tfc.imei.codimei =p_codimei;
   
   SELECT tfc.sosuportats.idso
   INTO soAplic
   FROM tfc.sosuportats
   WHERE soimei = tfc.sosuportats.idso
   AND idAplic = tfc.sosuportats.idaplicacio;
   
   
      EXCEPTION
        WHEN NO_DATA_FOUND THEN
        RAISE E_SO_NO_SOPORTADO;
      END;
   
   BEGIN
   SELECT tfc.imei.resoluciodispositiu
   INTO rd1
   FROM tfc.imei
   WHERE tfc.imei.codimei = p_codimei;
   
   SELECT tfc.aplicacions.resoluciminima
   INTO rd1OK
   FROM tfc.aplicacions
   WHERE idAplic = tfc.aplicacions.idaplicacio
   AND rd1 >= tfc.aplicacions.resoluciminima;
   
   EXCEPTION
        WHEN NO_DATA_FOUND THEN
        RAISE E_RESOLUCION_NO_SOPORTADA;
    END;
   
   BEGIN
   SELECT tfc.imei.resoluciodispositiu2
   INTO rd2
   FROM tfc.imei
   WHERE tfc.imei.codimei = p_codimei;
   
   SELECT tfc.aplicacions.resoluciominima2
   INTO rd2OK
   FROM tfc.aplicacions
   WHERE idAplic = tfc.aplicacions.idaplicacio
   AND rd1 >= tfc.aplicacions.resoluciominima2;
   
   EXCEPTION
        WHEN NO_DATA_FOUND THEN
        RAISE E_RESOLUCION_NO_SOPORTADA;
    END;
   
   BEGIN
   n_diadesc:= to_number(p_diadesc,'99');
   n_mesdesc:= to_number(p_mesdesc,'99');
   n_anodesc:= to_number(p_anodesc,'9999');
   
   IF( n_diadesc < 1 or n_diadesc > 31)
   THEN RAISE E_FECHA_DESCARGA_INVALIDA;
   END IF;
   
   IF( n_mesdesc < 1 or n_mesdesc > 12)
   THEN RAISE E_FECHA_DESCARGA_INVALIDA;
   END IF;
   IF( n_diadesc = 31 and (n_mesdesc=2 or n_mesdesc=4 or n_mesdesc=6
   or n_mesdesc=9 or n_mesdesc=11))
   THEN RAISE E_FECHA_DESCARGA_INVALIDA;
   END IF;
   IF( n_mesdesc = 2 and (n_diadesc=30 or n_diadesc=29))
   THEN RAISE E_FECHA_DESCARGA_INVALIDA;
   END IF;
   IF ((length (p_diadesc)<>2) OR (length (p_mesdesc)<>2) OR (length (p_anodesc)<>4)) 
   THEN RAISE E_FECHA_DESCARGA_INVALIDA;
   END IF;
   
   END;
   
   data_entrada := p_diadesc||p_mesdesc||p_anodesc;
   date_data_pujada:= to_date(data_entrada,'DDMMYYYY');
   
   SELECT MAX(factacumulada) 
   INTO factDescarregas
   FROM TFC.descarregas;
   
   SELECT COUNT(totaldescarregas) 
   INTO nDescarregas
   FROM TFC.descarregas;
   
   IF (factDescarregas IS NULL) THEN
   
   factDescarregas:= preuDescarrega;
   ELSE
      factDescarregas := preuDescarrega + factDescarregas;
   END IF;
   
   nDescarregas := nDescarregas + 1;
  
      INSERT INTO tfc.descarregas (codimei,idaplicacio,preuaplicacio,datadescarrega,ao,modepagament,factacumulada,totaldescarregas)
        VALUES (p_codimei,idAplic,preuDescarrega,date_data_pujada,n_anodesc,p_modepagament,factDescarregas ,nDescarregas);
        
        RSP := 'OK';

      INSERT INTO TFC.LOGS (datalog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
           VALUES(SYSDATE,'DESCARGAR_APLICACIO', param_entrada, 'RSP: '||RSP);

      COMMIT; 
      
       EXCEPTION
    WHEN E_APLICACION_NO_EXISTE THEN
      RSP := 'ERROR: La aplicacin no est de alta en el sistema';

      INSERT INTO TFC.LOGS (datalog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
         VALUES(SYSDATE, 'SP_DESCARGAR_APLICACIO', param_entrada, 'RSP: '||RSP);

      COMMIT;

  
    WHEN E_PAIS_NO_EXISTE THEN
      RSP := 'ERROR: Error al recuperar el pas';

      INSERT INTO TFC.LOGS (datalog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
         VALUES(SYSDATE, 'SP_DESCARGAR_APLICACIO', param_entrada, 'RSP: '||RSP);

      COMMIT;

  WHEN E_USER_NO_EXISTE THEN
      RSP := 'ERROR: Error al encontrar usuario por imei';

      INSERT INTO TFC.LOGS (datalog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
         VALUES(SYSDATE, 'SP_DESCARGAR_APLICACIO', param_entrada, 'RSP: '||RSP);

      COMMIT;


    WHEN E_CAMPO_NULO THEN
      RSP := 'ERROR: Alguno de los campos obligatorios es nulo';

      INSERT INTO TFC.LOGS (datalog, PROCEDIMENT, PARAMSENTRADA, PARAMSSortida)
         VALUES(SYSDATE, 'SP_DESCARGAR_APLICACIO', param_entrada, 'RSP: '||RSP);

      COMMIT;

    WHEN E_IMEI_NO_EXISTE THEN
      RSP := 'ERROR: El dispositivo asociado a ese IMEI no esxite o no tiene usuario';

      INSERT INTO TFC.LOGS (datalog, PROCEDIMENT, PARAMSENTRADA, PARAMSSortida)
         VALUES(SYSDATE, 'SP_DESCARGAR_APLICACIO', param_entrada, 'RSP: '||RSP);

      COMMIT;

    WHEN E_PREU_NO_EXISTE THEN
      RSP := 'ERROR: No existe esa aplicacin para el pas del usuario';

      INSERT INTO TFC.LOGS (datalog, PROCEDIMENT, PARAMSENTRADA, PARAMSSortida)
         VALUES(SYSDATE, 'SP_DESCARGAR_APLICACIO', param_entrada, 'RSP: '||RSP);

      COMMIT;
    WHEN E_SO_NO_SOPORTADO THEN
      RSP := 'ERROR: Aplicacin no disponible para el SO del dispositivo';

      INSERT INTO TFC.LOGS (datalog, PROCEDIMENT, PARAMSENTRADA, PARAMSSortida)
         VALUES(SYSDATE, 'SP_DESCARGAR_APLICACIO', param_entrada, 'RSP: '||RSP);

      COMMIT;

  
  WHEN E_RESOLUCION_NO_SOPORTADA THEN
      RSP := 'ERROR: Resolucin de la aplicacin no soportada por el dispositivo';

      INSERT INTO TFC.LOGS (datalog, PROCEDIMENT, PARAMSENTRADA, PARAMSSortida)
         VALUES(SYSDATE, 'SP_DESCARGAR_APLICACIO', param_entrada, 'RSP: '||RSP);

      COMMIT;
      
  WHEN E_FECHA_DESCARGA_INVALIDA THEN
      RSP := 'ERROR: FECHA O FORMATO DE FECHA DE DESCARGA NO VLIDA. FORMATO DD MM AAAA';

      INSERT INTO TFC.LOGS (datalog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
         VALUES(SYSDATE, 'SP_DESCARGAR_APLICACIO', param_entrada, 'RSP: '||RSP);

      COMMIT;

    WHEN OTHERS THEN
      err_num := SQLCODE;
      err_msg := SUBSTR(SQLERRM, 1, 200);

      RSP:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

      INSERT INTO tfc.LOGS (datalog, PROCEDIMENT, PARAMSENTRADA, paramssortida)
         VALUES(SYSDATE, 'SP_DESCARGAR_APLICACIO', param_entrada, 'RSP: '||RSP);

      COMMIT;


END SP_DESCARGAR_APLICACIO;
/
spool off;