SPOOL BD\24-logFU_LIST_DESARROLLADORES_PAIS.log

connect TFC/hola;
---------------
----LISTADO DE LOS DESARROLLADORES DE UN PAS
---------------
--EN ESTE TIPO SE VAN A INSERTAR LOS REGISTROS CON LOS DATOS DE LOS 
--DESARROLLADORES DE UN PAS DADO
drop type DESARROLLADORES_PAIS_OBJ force;

CREATE OR REPLACE TYPE DESARROLLADORES_PAIS_OBJ as object (
  nomempresa          VARCHAR2(20),
  numaplicacions      NUMBER,
  nomreplegal         VARCHAR2(20),
  cognom1replegal     VARCHAR2(20),
  cognom12replegal    VARCHAR2(20),
  adreaoficina       VARCHAR2(30),
  telefon            NUMBER
);
/
   
--Tipo que almacena todos los registros que devuelve el listado de desarrolladores 
--por pas
CREATE OR REPLACE TYPE  desarrolladores_pais_tab is table of DESARROLLADORES_PAIS_OBJ;             
/

CREATE OR REPLACE FUNCTION TFC.FU_LIST_DESARROLLADORES_PAIS
(p_codpais    IN desenvolupadors.codpaisregistre%TYPE)
RETURN desarrolladores_pais_tab
IS

  /******************************************************************************
   NOMBRE:      TFC.FU_LIST_DESARROLLADORES_PAIS
   PROPSITO:   Funcin que devuelve un listado con todos los desarrolladores de un pas
                que se le pasa como parmetro
                  -Nombre de la empresa         
                  -Nombre del representante legal
                  -Primer apellido representante legal           
                  -Segundo apellido representante legal
                  -Direccin de la oficina    
                  -Telfono
                  
                 Una vez finalizado inserta en la tabla de Log la el resultado de la ejecucion, OK si 
                 ha ido bien o el Error en caso contrario
   PARAMETROS:
        INPUT: p_codpais: cdigo del pas
        OUTPUT: RSP
   RETORNA:
        Si ha ido bien retorna un Listado con todos los desarrolladores del pas pasado como parametro
        Si no ha ido bien alguno de los mensajes siguientes, dependiendo del error:
                'ERROR: El pas es Obligatorio, ejecutar FU_LIST_DESARROLLADORES_PAS(CODPAIS)'
                'ERROR: Tipo de Error' en cualquier otro caso
   EJECUCION:
       SELECT * FROM TABLE(FU_LIST_DESARROLLADORES_PAIS('ES'));
   ******************************************************************************/

   L_DESARROLLADORES_PAIS_TAB DESARROLLADORES_PAIS_TAB:= DESARROLLADORES_PAIS_TAB();
   E_CAMPO_NULO      EXCEPTION;  
   N                 NUMBER := 0;
   RSP               VARCHAR2(2000);
   err_num           NUMBER;
   err_msg           VARCHAR2(100);
   param_entrada     VARCHAR2(500);
   param_sortida     VARCHAR2(100);
   NUMAPLICACIONS   NUMBER;

   CURSOR CUR_DES_PAIS(pid_pais DESENVOLUPADORS.CODPAISREGISTRE%TYPE) IS
     SELECT dv.nomempresa,count(dvm.idaplicacio)AS NUMAPLICACIONS,dv.nomreplegal,dv.cognom1replegal,dv.cognom2replegal,dv.adreaoficina,dv.telefon 
  FROM desenvolupadors dv JOIN desenvolupaments dvm ON dv.iddesenvolupador = dvm.iddesenvolupador
  WHERE dv.codpaisregistre = pid_pais
  GROUP BY dv.nomempresa,dv.nomreplegal,dv.cognom1replegal,dv.cognom2replegal,dv.adreaoficina,dv.telefon;

    
  PRAGMA AUTONOMOUS_TRANSACTION;
    
BEGIN

  param_entrada := 'P_CODPAIS: ' ||p_codpais;
  
  --Verifica que todos los campos obligatorios tengan valor
  IF (TRIM(p_codpais) IS NULL)
  THEN
      RAISE E_CAMPO_NULO;
  END IF;

                
  FOR REG1 IN CUR_DES_PAIS(p_codpais) LOOP
    L_DESARROLLADORES_PAIS_TAB.EXTEND;
    N := N + 1;
    L_DESARROLLADORES_PAIS_TAB(N) := DESARROLLADORES_PAIS_OBJ(REG1.NOMEMPRESA, REG1.NUMAPLICACIONS, REG1.NOMREPLEGAL, REG1.COGNOM1REPLEGAL, REG1.COGNOM2REPLEGAL, 
                                                REG1.ADREAOFICINA,REG1.TELEFON);    
  END LOOP;
  
  RSP := 'OK';  

  INSERT INTO LOGS (DATALOG, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
       VALUES(SYSDATE,'FU_LIST_DESARROLLADORES_PAIS', param_entrada, 'RSP: '||RSP);
  
  COMMIT;
  
  RETURN L_DESARROLLADORES_PAIS_TAB;

  
EXCEPTION
  WHEN E_CAMPO_NULO THEN
     RSP := 'ERROR: El pais es Obligatorio, ejecutar FU_LIST_DESARROLLADORES_PAIS(PAIS)';

     INSERT INTO LOGS (DATALOG, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
        VALUES(SYSDATE, 'FU_DESARROLLADORES_PAIS_CURSO', param_entrada, 'RSP: '||RSP);

  WHEN OTHERS THEN
     err_num := SQLCODE;
     err_msg := SUBSTR(SQLERRM, 1, 200);

     RSP:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

     INSERT INTO LOGS (DATALOG, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
        VALUES(SYSDATE, 'FU_DESARROLLADORES_PAIS_CURSO', param_entrada, 'RSP: '||RSP);

     COMMIT;
     
  
END;
/
spool off;