SPOOL BD\25-logFU_LIST_APLICACIONES_DESC.log

connect TFC/hola;
---------------
----LISTADO DE LOS APLICACIONES ACTIVAS Y DATOS PRINNCIPALES
---------------
--EN ESTE TIPO SE VAN A INSERTAR LOS REGISTROS CON LOS DATOS DE LAS 
--APLICACIONES ACTIVAS ORDENADAS POR NMERO DE DESCARGA
drop type LIST_APLICACIONES_OBJ force;

CREATE OR REPLACE TYPE LIST_APLICACIONES_OBJ as object (
  numdescarregas        NUMBER,
  nomaplicacio          VARCHAR2(40),
  nomso                 VARCHAR2(20),
  codpais               VARCHAR2(2),
  preuAplicacio         NUMBER,
  descripcio            VARCHAR2(100)
  
);
/
   
--Tipo que almacena todos los registros que devuelve el listado de desarrolladores 
--por pas
CREATE OR REPLACE TYPE  LIST_APLICACIONES_TAB is table of LIST_APLICACIONES_OBJ;             
/

CREATE OR REPLACE FUNCTION TFC.FU_LIST_APLICACIONES_DESC
RETURN LIST_APLICACIONES_TAB
IS

  /******************************************************************************
   NOMBRE:      TFC.FU_LIST_APLICACIONES_DESC
   PROPSITO:   Funcin que devuelve un listado con todos las aplicaiones descargadas
                con sus datos principales ordenandas por nmero de descarga. En caso
                de que la aplicacion tenga varias versiones o este disponible en varios paises
                se mostraran los datos por versin y pais pero con el total absiluto de descargas.
                    -Nmero de descargas
                    -Nombre de la aplicacin
                    -SO soportado por la aplicacin
                    -Versin de la aplicacin
                    -Pais donde se oferta la aplicacin
                    -url del Video
                    -resolucin mnima ancho
                    -resolucin mnima alto
                    -Si est activa
                    -Fecha de subida al sistema
                    -precio de la aplicain (por Pas)
                    -Descripcin de la aplicacin (Por pas)
                    -url de la Aplicacin (por SO Soportado)
                    -Tamao de la aplicacin (por SO Soportado) 
                 Una vez finalizado inserta en la tabla de Log la el resultado de la ejecucion, OK si 
                 ha ido bien o el Error en caso contrario
   PARAMETROS:
        OUTPUT: RSP
DEVUELVE:
        Si ha ido bien decuelve una lista de las aplicaciones actrivas ordenadas por nmero de descargas
        Si no ha ido bien alguno de los mensajes siguientes, dependiendo del error:
                'ERROR: Tipo de Error' en cualquier otro caso
   EJECUCION:
       SELECT * FROM TABLE(FU_LIST_APLICACIONES_DESC);
   ******************************************************************************/

   L_LIST_APLICACIONES_TAB LIST_APLICACIONES_TAB:= LIST_APLICACIONES_TAB();
   E_CAMPO_NULO      EXCEPTION;  
   N                 NUMBER := 0;
   RSP               VARCHAR2(2000);
   err_num           NUMBER;
   err_msg           VARCHAR2(100);
   param_entrada     VARCHAR2(1000);
   param_sortida     VARCHAR2(200);
   

   CURSOR CUR_LIST_APLICACIONES IS
  SELECT COUNT (dc.idaplicacio)AS NumDescarregas,ap.nomaplicacio,so.nomso,
pa.codpais,pa.preuaplicacio,pa.descripcio
FROM (((descarregas dc INNER JOIN aplicacions ap ON dc.idaplicacio = ap.idaplicacio)
INNER JOIN sosuportats ss ON ap.idaplicacio = ss.idaplicacio)
INNER JOIN preusaplicacions pa ON ap.idaplicacio = pa.idaplicacio)
INNER JOIN sistemasoperatius so ON so.idso = ss.idso
GROUP BY dc.idaplicacio, ap.nomaplicacio,
ss.idso,so.nomso,pa.codpais,pa.preuaplicacio,pa.descripcio
order BY count(dc.idaplicacio) DESC,ss.idso ASC;
    
  PRAGMA AUTONOMOUS_TRANSACTION;
    
BEGIN

  FOR REG1 IN CUR_LIST_APLICACIONES LOOP
    L_LIST_APLICACIONES_TAB.EXTEND;
    N := N + 1;
    L_LIST_APLICACIONES_TAB(N) := LIST_APLICACIONES_OBJ(REG1.NUMDESCARREGAS, REG1.NOMAPLICACIO,REG1.NOMSO,
                                  REG1.CODPAIS,REG1.PREUAPLICACIO,REG1.DESCRIPCIO);
  END LOOP;
  
  RSP := 'OK';  

  INSERT INTO LOGS (DATALOG, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
       VALUES(SYSDATE,'FU_LIST_APLICACIONES', param_entrada, 'RSP: '||RSP);
  
  COMMIT;
  
  RETURN L_LIST_APLICACIONES_TAB;

  
EXCEPTION
  
  WHEN OTHERS THEN
     err_num := SQLCODE;
     err_msg := SUBSTR(SQLERRM, 1, 200);

     RSP:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

     INSERT INTO LOGS (DATALOG, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
        VALUES(SYSDATE, 'FU_LIST_APLICACIONES', param_entrada, 'RSP: '||RSP);

     COMMIT;
     
  
END;
/
spool off;