SPOOL BD\26-logFU_LIST_APLIC_ANO_PAIS.log

connect TFC/hola;
---------------
----LISTADO DE LOS PAISES Y N DE DESCARGAS
---------------
--EN ESTE TIPO SE VAN A INSERTAR LOS REGISTROS CON LOS DATOS DE LOS 
--PAISES Y N DE DESCARGAS
DROP TYPE APLIC_ANO_PAIS_OBJ FORCE;

CREATE OR REPLACE TYPE APLIC_ANO_PAIS_OBJ as object (
  paisusuari         VARCHAR2(2),
  numdescargas      NUMBER
);
/
   
--Tipo que almacena todos los registros que devuelve el listado de desarrolladores 
--por pas
CREATE OR REPLACE TYPE  APLIC_ANO_PAIS_TAB is table of APLIC_ANO_PAIS_OBJ;             
/

CREATE OR REPLACE FUNCTION TFC.FU_LIST_APLIC_ANO_PAIS
(p_nomaplicacio IN aplicacions.nomaplicacio%TYPE,
p_ano    IN NUMBER)
RETURN APLIC_ANO_PAIS_TAB
IS

  /******************************************************************************
   NOMBRE:      TFC.FU_LIST_APLIC_ANO_PAIS
   PROPSITO:   Funcin que devuelve un listado con el nmero de descargas por pais
   para una aplicacin y ao dados
                  -Nmero de descargas         
                  -Pas de la descarga
                  
                 Una vez finalizado inserta en la tabla de Log la el resultado de la ejecucion, OK si 
                 ha ido bien o el Error en caso contrario
   PARAMETROS:
        INPUT: p_nomaplicacio: nombre de la aplicacin,p_ano: ao para el que consultamos 
        OUTPUT: RSP
   RETORNA:
        Si ha ido bien retorna un Listado con el nmero de descargas por pas para esa aplicacin y ao
        Si no ha ido bien alguno de los mensajes siguientes, dependiendo del error:
                'ERROR: El nombre o ao son Obligatorio, ejecutar FU_LIST_APLIC_ANO_PAIS(NOMAPLICACIO,AO)'
                'ERROR: Tipo de Error' en cualquier otro caso
   EJECUCION:
       SELECT * FROM TABLE(FU_LIST_APLIC_ANO_PAIS('Informix','2012'));
   ******************************************************************************/

   L_APLIC_ANO_PAIS_TAB APLIC_ANO_PAIS_TAB:= APLIC_ANO_PAIS_TAB();
   E_CAMPO_NULO      EXCEPTION;  
   N                 NUMBER := 0;
   RSP               VARCHAR2(2000);
   err_num           NUMBER;
   err_msg           VARCHAR2(100);
   param_entrada     VARCHAR2(500);
   param_sortida     VARCHAR2(100);
   NUMAPLICACIONS   NUMBER;

   CURSOR CUR_DES_ANO(pid_nomaplicacio aplicacions.nomaplicacio%TYPE,pid_ano NUMBER) IS
    SELECT  usr.paisusuari,COUNT( (dc.idaplicacio)) AS NumDescargas
FROM ((aplicacions ap INNER JOIN descarregas dc ON ap.idaplicacio =dc.idaplicacio
INNER JOIN imei im ON dc.codimei = im.codimei)
INNER JOIN usuaris usr ON im.idusuariimei = usr.idusuari)
WHERE (extract(year from dc.datadescarrega) = pid_ano AND ap.nomaplicacio = pid_nomaplicacio)
GROUP BY dc.datadescarrega,usr.paisusuari;

    
  PRAGMA AUTONOMOUS_TRANSACTION;
    
BEGIN

  param_entrada := 'P_NOMAPLICACIO: ' ||p_nomaplicacio||',
     P_ANO:'||p_ano;
    
  
  --Verifica que todos los campos obligatorios tengan valor
  IF (TRIM(p_nomaplicacio) IS NULL OR TRIM(p_ano) IS NULL)
  THEN
      RAISE E_CAMPO_NULO;
  END IF;

                
  FOR REG1 IN CUR_DES_ANO(p_nomaplicacio,p_ano) LOOP
    L_APLIC_ANO_PAIS_TAB.EXTEND;
    N := N + 1;
    L_APLIC_ANO_PAIS_TAB(N) := APLIC_ANO_PAIS_OBJ(REG1.PAISUSUARI, REG1.NUMDESCARGAS);    
  END LOOP;
  
  RSP := 'OK';  

  INSERT INTO LOGS (DATALOG, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
       VALUES(SYSDATE,'FU_LIST_APLIC_ANO_PAIS', param_entrada, 'RSP: '||RSP);
  
  COMMIT;
  
  RETURN L_APLIC_ANO_PAIS_TAB;

  
EXCEPTION
  WHEN E_CAMPO_NULO THEN
     RSP := 'ERROR: La aplicacin o el ao son obligatorios, ejecutar FU_LIST_APLIC_ANO_PAIS(''Aplicacin'',''Ao'')';

     INSERT INTO LOGS (DATALOG, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
        VALUES(SYSDATE, 'FU_APLIC_ANO_PAIS', param_entrada, 'RSP: '||RSP);

  WHEN OTHERS THEN
     err_num := SQLCODE;
     err_msg := SUBSTR(SQLERRM, 1, 200);

     RSP:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

     INSERT INTO LOGS (DATALOG, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
        VALUES(SYSDATE, 'FU_APLIC_ANO_PAIS', param_entrada, 'RSP: '||RSP);

     COMMIT;
     
  
END;
/
spool off;