SPOOL BD\27-logFU_LIST_ACTIVIDAD_USR.log

connect TFC/hola;
---------------
----LISTADO DE LOS ACTIVIDAD DE DESCARGA DE UN USUARIO
---------------
--EN ESTE TIPO SE VAN A INSERTAR LOS REGISTROS CON LOS DATOS DE LA 
--ACTIVIDAD DE DESCARGA PARA UN USUARIO DADO

DROP TYPE ACTIVIDAD_USR_OBJ FORCE;
CREATE OR REPLACE TYPE ACTIVIDAD_USR_OBJ as object (
  datadescarrega         DATE,
  nomaplicacio           VARCHAR2(40),
  preuaplicacio          NUMBER,
  modepagament           VARCHAR2(10)
  );
/
   
--Tipo que almacena todos los registros que devuelve el listado de desarrolladores 
--por pas
CREATE OR REPLACE TYPE  ACTIVIDAD_USR_TAB is table of ACTIVIDAD_USR_OBJ;             
/

CREATE OR REPLACE FUNCTION TFC.FU_LIST_ACTIVIDAD_USR
(p_nummobil IN usuaris.nummobil%TYPE)
RETURN ACTIVIDAD_USR_TAB
IS

  /******************************************************************************
   NOMBRE:      TFC.FU_LIST_ACTIVIDAD_USR
   PROPSITO:   Funcin que devuelve un listado la actividad de descargas por usuario
 en la plataforma por fecha y aplicacin de descarga
                  -Fecha de descarga
                  -Nombre de aplicacin de descarga
                  -Precio de la aplicacin
                  -Modo de pago
                  
                 Una vez finalizado inserta en la tabla de Log  el resultado de la ejecucion, OK si 
                 ha ido bien o el Error en caso contrario
   PARAMETROS:
        INPUT: p_nummobil: nmero de mvil del usuario para el que queremos ver su actividad
        OUTPUT: RSP
   RETORNA:
        Si ha ido bien retorna un Listado con el histrico de actividades en la plataforma de descargas
        Si no ha ido bien alguno de los mensajes siguientes, dependiendo del error:
                'ERROR: El nmero de telfono es obligatorio, ejecutar FU_LIST_ACTIVIDAD_USR_PAIS(NOMAPLICACIO,AO)'
                'ERROR: Tipo de Error' en cualquier otro caso
   EJECUCION:
       SELECT * FROM TABLE(FU_LIST_ACTIVIDAD_USR('616634669'));
   ******************************************************************************/

   L_ACTIVIDAD_USR_TAB ACTIVIDAD_USR_TAB:= ACTIVIDAD_USR_TAB();
   E_CAMPO_NULO      EXCEPTION;  
   N                 NUMBER := 0;
   RSP               VARCHAR2(2000);
   err_num           NUMBER;
   err_msg           VARCHAR2(100);
   param_entrada     VARCHAR2(500);
   param_sortida     VARCHAR2(100);
   NUMAPLICACIONS   NUMBER;

   CURSOR CUR_ACT_TLF(pid_nummobil usuaris.nummobil%TYPE) IS
    SELECT DISTINCT ap.nomaplicacio,dc.datadescarrega, dc.preuaplicacio,dc.modepagament 
FROM (((descarregas dc INNER JOIN preusaplicacions pa ON dc.idaplicacio = pa.idaplicacio)
INNER JOIN imei im ON dc.codimei = im.codimei)
INNER JOIN usuaris us ON us.idusuari =im.idusuariimei)
INNER JOIN aplicacions ap ON dc.idaplicacio = ap.idaplicacio
WHERE us.nummobil = pid_nummobil;
    
  PRAGMA AUTONOMOUS_TRANSACTION;
    
BEGIN

  param_entrada := 'P_NUMMOBIL: ' ||p_nummobil;
    
  
  --Verifica que todos los campos obligatorios tengan valor
  IF (TRIM(p_nummobil) IS NULL)
  THEN
      RAISE E_CAMPO_NULO;
  END IF;

                
  FOR REG1 IN CUR_ACT_TLF(p_nummobil) LOOP
    L_ACTIVIDAD_USR_TAB.EXTEND;
    N := N + 1;
    L_ACTIVIDAD_USR_TAB(N) := ACTIVIDAD_USR_OBJ(REG1.DATADESCARREGA, REG1.NOMAPLICACIO,REG1.PREUAPLICACIO,REG1.MODEPAGAMENT);    
  END LOOP;
  
  RSP := 'OK';  

  INSERT INTO LOGS (DATALOG, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
       VALUES(SYSDATE,'FU_LIST_ACTIVIDAD_USR', param_entrada, 'RSP: '||RSP);
  
  COMMIT;
  
  RETURN L_ACTIVIDAD_USR_TAB;

  
EXCEPTION
  WHEN E_CAMPO_NULO THEN
     RSP := 'ERROR: El nmero de telfono es obligatorio, ejecutar FU_LIST_ACTIVIDAD_USR(''Telefono'')';

     INSERT INTO LOGS (DATALOG, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
        VALUES(SYSDATE, 'FU_LIST_ACTIVIDAD_USR', param_entrada, 'RSP: '||RSP);

  WHEN OTHERS THEN
     err_num := SQLCODE;
     err_msg := SUBSTR(SQLERRM, 1, 200);

     RSP:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

     INSERT INTO LOGS (DATALOG, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
        VALUES(SYSDATE, 'FU_LIST_ACTIVIDAD_USR', param_entrada, 'RSP: '||RSP);

     COMMIT;
     
  
END;
/
spool off;