SPOOL BD\28-logFU_TOP20_USR.log

connect TFC/hola;
---------------
----LISTADO DE LOS 20 USUARIOS CON MAYOR FACTURACIN DE DESCARGA PARA UN AO DADO
---------------
--EN ESTE TIPO SE VAN A INSERTAR LOS REGISTROS CON LOS DATOS DE LOS 20
--USUARIOS CON MAYOR FACTURACIN DE DESCARGAS PARA UN AO DADO
DROP TYPE TOP20_USR_OBJ FORCE;

CREATE OR REPLACE TYPE TOP20_USR_OBJ as object (
  nummobil               NUMBER,
  nomusuari              VARCHAR2(20),
  cognom1usuari          VARCHAR2(20),
  cognom2usuari          VARCHAR2(20),
  facturacion            NUMBER
  );
/
   
--Tipo que almacena todos los registros que devuelve el listado de desarrolladores 
--por pas
CREATE OR REPLACE TYPE  TOP20_USR_TAB is table of TOP20_USR_OBJ;             
/

CREATE OR REPLACE FUNCTION TFC.FU_TOP20_USR
(p_ano IN NUMBER)
RETURN TOP20_USR_TAB
IS

  /******************************************************************************
   NOMBRE:      TFC.FU_TOP20_USR
   PROPSITO:   Funcin que devuelve un listado de los 20 usuarios que ms han
   gastado en aplicaciones para una ao dado
                  -telefono
                  -Nombre del usuairo
                  -Primer apellido del usuario
                  -Segundo apellido del usuario
                  -Facturacin
                 Una vez finalizado inserta en la tabla de Log  el resultado de la ejecucion, OK si 
                 ha ido bien o el Error en caso contrario
   PARAMETROS:
        INPUT: p_ano: ao para el que qeuremos hacer la consulta
        OUTPUT: RSP
   RETORNA:
        Si ha ido bien retorna un Listado un listado de los 20 usuarios que ms han
   gastado en aplicaciones para una ao dado
        Si no ha ido bien alguno de los mensajes siguientes, dependiendo del error:
                'ERROR: El ao es obligatorio, ejecutar FU_TOP20_USR(AO)'
                'ERROR: Tipo de Error' en cualquier otro caso
   EJECUCION:
       SELECT * FROM TABLE(FU_TOP20_USR('2012'));
   ******************************************************************************/

   L_TOP20_USR_TAB TOP20_USR_TAB:= TOP20_USR_TAB();
   E_CAMPO_NULO      EXCEPTION;  
   N                 NUMBER := 0;
   RSP               VARCHAR2(2000);
   err_num           NUMBER;
   err_msg           VARCHAR2(100);
   param_entrada     VARCHAR2(500);
   param_sortida     VARCHAR2(100);
   NUMAPLICACIONS   NUMBER;

   CURSOR CUR_TOP20(pid_ano NUMBER) IS
    SELECT * FROM
(select us.nummobil,us.nomusuari,us.cognom1usuari,us.cognom2usuari,sum(dc.preuaplicacio) AS facturacion
from descarregas dc,imei im,usuaris us
WHERE (dc.codimei=im.codimei) AND
(us.idusuari = im.idusuariimei) AND
extract(year from dc.datadescarrega) = p_ano
group by us.nummobil,us.nomusuari,us.cognom1usuari,us.cognom2usuari,dc.preuaplicacio
ORDER BY SUM(dc.preuaplicacio) DESC )
WHERE ROWNUM < 21;
    
 PRAGMA AUTONOMOUS_TRANSACTION;
    
BEGIN

  param_entrada := 'P_ANO: ' ||p_ano;
    
  
  --Verifica que todos los campos obligatorios tengan valor
  IF (TRIM(p_ano) IS NULL)
  THEN
      RAISE E_CAMPO_NULO;
  END IF;

                
  FOR REG1 IN CUR_TOP20(p_ano) LOOP
    L_TOP20_USR_TAB.EXTEND;
    N := N + 1;
    L_TOP20_USR_TAB(N) := TOP20_USR_OBJ(REG1.NUMMOBIL, REG1.NOMUSUARI,REG1.COGNOM1USUARI,REG1.COGNOM2USUARI,REG1.FACTURACION);    
  END LOOP;
  
  RSP := 'OK';  

  INSERT INTO LOGS (DATALOG, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
       VALUES(SYSDATE,'FU_TOP20_USR', param_entrada, 'RSP: '||RSP);
  
  COMMIT;
  
  RETURN L_TOP20_USR_TAB;

  
EXCEPTION
  WHEN E_CAMPO_NULO THEN
     RSP := 'ERROR: El ao es obligatorio, ejecutar FU_TOP20_USR(''Ao'')';

     INSERT INTO LOGS (DATALOG, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
        VALUES(SYSDATE, 'FU_TOP20_USR', param_entrada, 'RSP: '||RSP);

  WHEN OTHERS THEN
     err_num := SQLCODE;
     err_msg := SUBSTR(SQLERRM, 1, 200);

     RSP:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

     INSERT INTO LOGS (DATALOG, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
        VALUES(SYSDATE, 'FU_TOP20_USR', param_entrada, 'RSP: '||RSP);

     COMMIT;
     
  
END;
/
spool off;