spool BD\9-logSP_ALTA_USUARI.log;

CONNECT TFC/hola;

create or replace PROCEDURE SP_ALTA_USUARI (
   p_nummobil 	IN  TFC.USUARIS.NUMMOBIL%TYPE,
  
   p_nomusuari  	IN  TFC.USUARIS.NOMUSUARI%TYPE,
   p_cognom1usuari  	IN  TFC.USUARIS.COGNOM1USUARI%TYPE,
   p_cognom2usuari  	IN  TFC.USUARIS.COGNOM2USUARI%TYPE,
   p_paisusuari  	IN  TFC.USUARIS.PAISUSUARI%TYPE,
   p_operadortelefonic  	IN  TFC.USUARIS.OPERADORTELEFONIC%TYPE,
   p_emailusuari		IN  TFC.USUARIS.EMAILUSUARI%TYPE,
   p_codimei    IN TFC.imei.CODIMEI%TYPE,
   
   RSP         		OUT VARCHAR2
)
IS
/******************************************************************************
   Nombre:                 SP_ALTA_USUARI
   Propsito:            introduce un usuario a la tabla USUARIS
                   
   Parmetros entrada: 
                   -   p_nomusuari  	 VARCHAR2(20) , nombre del usuario
                   -   p_cognom1usuari VARCHAR2(20) , primer apellido usuario
                   -   p_cognom2usuari VARCHAR2(20) , segundo apellido usuario
                   -   p_paisusuari  	VARCHAR2(2), Cdigo pas del usuario
                   -   p_operadortelefonic VARCHAR2(20) , operador telefnico
                  -    p_emailusuari		VARCHAR2(40) , email usuario
                  -    p_codimei    VARCHAR2(15), primer imei usuario,                                        
   Parmetros salida:  RSP
   Retorna:             'OK' si la ejecucin ha finalizado con xito
                        'ERROR: tipo de error' si se ha producido un error y 
			dependiendo del error puede ser
			
******************************************************************************/

   E_CAMPO_NULO         EXCEPTION;
   E_USUARIO_EXISTE      EXCEPTION;
   E_PAIS_NO_EXISTE EXCEPTION;
   E_IMEI_NO_EXISTE EXCEPTION;

   existe        VARCHAR2(1);
   existepais        VARCHAR2(1);
   existeimei        VARCHAR2(1);
   param_entrada VARCHAR2(500);
   err_num       NUMBER;
   err_msg       VARCHAR2(100);
   p_idusuari NUMBER;

   BEGIN

     param_entrada := 'P_NUMMOBIL: '||p_nummobil||', P_NOMUSUARI:'||p_nomusuari||', P_COGNOM1USUARI:'||p_cognom1usuari||', P_COGNOM2USUARI:'||p_cognom2usuari||', P_PAISUSUARI:'||p_paisusuari||', P_OPERADORTELEFONIC:'||p_operadortelefonic||',  P_EMAILUSUARI:'||p_emailusuari||', P_CODIMEI:'||p_codimei;

     --Verifica que todos los campos obligatorios tengan valor
     IF (TRIM(p_nummobil)     IS NULL)
     THEN
         RAISE E_CAMPO_NULO;
      END IF;

     --Verifica que el registro no se haya insertado previmente con otro ID
     BEGIN

       SELECT 'S'
         INTO EXISTE
         FROM TFC.USUARIS
        WHERE NUMMOBIL  = p_nummobil
          AND ROWNUM  = 1;

        RAISE E_USUARIO_EXISTE;
      EXCEPTION
        WHEN NO_DATA_FOUND THEN
           NULL;
      END;

       BEGIN
       SELECT 'S'
         INTO EXISTEPAIS
         FROM TFC.PAISOS
        WHERE CODPAIS  = p_paisusuari
          AND ROWNUM  = 1;
        EXCEPTION
        WHEN NO_DATA_FOUND THEN
          RAISE  E_PAIS_NO_EXISTE;
           
           
           
      END;
BEGIN
       SELECT 'S'
         INTO EXISTEimei
         FROM TFC.IMEI
        WHERE CODIMEI  = p_codimei
          AND ROWNUM  = 1;
        EXCEPTION
        WHEN NO_DATA_FOUND THEN
          RAISE  E_IMEI_NO_EXISTE;
           
           
           
      END;
      --Si ha pasado todas las pruebas de validacin realiza la insercin

  
      INSERT INTO TFC.USUARIS (nummobil,nomusuari,cognom1usuari,cognom2usuari,paisusuari,operadortelefonic,emailusuari)
           VALUES (p_nummobil, p_nomusuari,p_cognom1usuari,p_cognom2usuari,p_paisusuari,p_operadortelefonic,p_emailusuari);
      
      SELECT TFC.usuaris.IDUSUARI 
      INTO p_idusuari
      FROM TFC.USUARIS
      WHERE NUMMOBIL  = p_nummobil;
      
      
      UPDATE TFC.imei
      set idusuariimei = p_idusuari
      where tfc.imei.codimei = p_codimei;
      RSP := 'OK';

   

      INSERT INTO TFC.LOGS (datalog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
           VALUES(SYSDATE,'SP_ALTA_USUARI', param_entrada, 'RSP: '||RSP);

      COMMIT;

  EXCEPTION
    WHEN E_USUARIO_EXISTE THEN
      RSP := 'ERROR: EL USUARIO YA EST DADO DE ALTA EN EL SISTEMA';

      INSERT INTO TFC.LOGS (datalog, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
         VALUES(SYSDATE, 'SP_ALTA_USUARI', param_entrada, 'RSP: '||RSP);

      COMMIT;

    WHEN E_CAMPO_NULO THEN
      RSP := 'ERROR: EL CAMPO DEL NMERO DE MVIL EST EN BLANCO';

      INSERT INTO TFC.LOGS (datalog, PROCEDIMENT, PARAMSENTRADA, PARAMSSortida)
         VALUES(SYSDATE, 'SP_ALTA_USUARI', param_entrada, 'RSP: '||RSP);

      COMMIT;

    WHEN E_PAIS_NO_EXISTE THEN
      RSP := 'ERROR: EL PAS NO EXISTE';

      INSERT INTO LOGS (DATALOG, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
         VALUES(SYSDATE, 'SP_ALTA_USUARI', param_entrada, 'RSP: '||RSP);

      COMMIT;

    WHEN E_IMEI_NO_EXISTE THEN
      RSP := 'ERROR: EL IMEI NO EXISTE';

      INSERT INTO LOGS (DATALOG, PROCEDIMENT, PARAMSENTRADA, PARAMSSORTIDA)
         VALUES(SYSDATE, 'SP_ALTA_USUARI', param_entrada, 'RSP: '||RSP);

      COMMIT;

    WHEN OTHERS THEN
      err_num := SQLCODE;
      err_msg := SUBSTR(SQLERRM, 1, 200);

      RSP:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

      INSERT INTO tfc.LOGS (datalog, PROCEDIMENT, PARAMSENTRADA, paramssortida)
         VALUES(SYSDATE, 'SP_ALTA_DESENVOLUPADOR', param_entrada, 'RSP: '||RSP);

      COMMIT;


END SP_ALTA_USUARI;
/
spool off;